package com.softslate.commerce.businessobjects.core;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Locale;
import java.util.Map;
import java.util.Properties;

import javax.inject.Inject;
import javax.inject.Named;

import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.google.common.eventbus.EventBus;
import com.google.inject.Injector;
import com.softslate.commerce.businessobjects.order.Order;
import com.softslate.commerce.businessobjects.order.OrderDiscount;
import com.softslate.commerce.businessobjects.order.OrderItem;
import com.softslate.commerce.businessobjects.order.OrderItemAttribute;
import com.softslate.commerce.daos.core.DAOFactory;
import com.softslate.commerce.daos.product.ProductGatewayDAO;

/**
 * Abstract superclass for all processing objects in the business layer.
 * (Classes that handle business processing are all named with the suffix
 * "Processor".)
 * <p>
 * The methods provided here allow subclasses to communicate with other elements
 * of the application. In particular, retrieve general settings from the
 * database (using {@link #getSettings()}), retrive application settings from
 * the <code>/WEB-INF/classes/appSettings.properties</code> file (using
 * {@link #getAppSettings()}, retrive application components from the
 * <code>/WEB-INF/classes/appComponents.properties</code> file (using
 * {@link #getAppComponents()}, retrieve user information (using
 * {@link #getUser()}), create data access objects (using
 * {@link #getDaoFactory()}) and create other business objects (using
 * {@link #getBusinessObjectFactory()}).
 * 
 * @author David Tobey
 * 
 */
public abstract class BaseBusinessProcessor extends BaseBusinessObject
		implements BusinessProcessor {

	static Log log = LogFactory.getLog(BaseBusinessProcessor.class);

	private Settings settings = null;

	public Settings getSettings() {
		return (this.settings);
	}

	@Inject
	public void setSettings(Settings settings) {
		this.settings = settings;
	}

	private Properties appSettings = null;

	public Properties getAppSettings() {
		return appSettings;
	}

	@Inject
	public void setAppSettings(@Named("appSettings") Properties appSettings) {
		this.appSettings = appSettings;
	}

	private Properties appComponents = null;

	public Properties getAppComponents() {
		return appComponents;
	}

	@Inject
	public void setAppComponents(@Named("appComponents") Properties appComponents) {
		this.appComponents = appComponents;
	}

	private User user = null;

	public User getUser() {
		return (this.user);
	}

	@Inject
	public void setUser(User user) {
		this.user = user;
	}

	private DAOFactory daoFactory = null;

	public DAOFactory getDaoFactory() {
		return (this.daoFactory);
	}

	@Inject
	public void setDaoFactory(DAOFactory daoFactory) {
		this.daoFactory = daoFactory;
	}

	public void initialize(Map transferMap) throws Exception {
		PropertyUtils.copyProperties(this, transferMap);
		return;
	}

	private BusinessObjectFactory businessObjectFactory = null;

	public BusinessObjectFactory getBusinessObjectFactory() {
		return (this.businessObjectFactory);
	}

	@Inject
	public void setBusinessObjectFactory(
			BusinessObjectFactory businessObjectFactory) {
		this.businessObjectFactory = businessObjectFactory;
	}

	private BusinessObjectUtils utils = null;

	public BusinessObjectUtils utils() {
		if (utils != null) {
			return utils;
		}
		utils = (BusinessObjectUtils) getBusinessObjectFactory()
				.createUtilsObject("businessObjectUtilsImplementer");
		return utils;
	}

	private Injector injector = null;

	public Injector getInjector() {
		return injector;
	}

	@Inject
	public void setInjector(Injector injector) {
		this.injector = injector;
	}

	private EventBus eventBus = null;

	public EventBus getEventBus() {
		return eventBus;
	}

	@Inject
	public void setEventBus(EventBus eventBus) {
		this.eventBus = eventBus;
	}

	/**
	 * Formats a Date object into a string suitable for storage in the database.
	 * 
	 * @param dateTime
	 * @return a String in the form of "yyyy-MM-dd HH:mm:ss:SSS" suitable for
	 *         storage in the database.
	 */
	public String formatDateTime(Date dateTime) {
		return utils().formatDateTime(dateTime);
	}

	/**
	 * Parses a string into a Date object.
	 * 
	 * @param dateTime
	 *            a String in the form of "yyyy-MM-dd HH:mm:ss:SSS" suitable for
	 *            storage in the database.
	 * @return a Date object.
	 */
	public Date parseDateTime(String dateTime) {
		return utils().parseDateTime(dateTime);
	}

	/**
	 * Formats a double into a string suitable for displaying the double as a
	 * price.
	 * 
	 * @param rawPrice
	 * @param currencyCode
	 * @param locale
	 * @return a String suitable for displaying the double as a price.
	 */
	public String formatPrice(double rawPrice, String currencyCode,
			Locale locale) {
		return utils().formatPrice(rawPrice, currencyCode, locale);
	}

	/**
	 * Prepares a url-encoded string for use with an HTTP request, given a map
	 * of the request's parameters.
	 * 
	 * @param params
	 *            A Map of key-value String pairs representing the parameters of
	 *            the request.
	 * @return A String representing a valid URL-encoded query string.
	 * @throws Exception
	 */
	public String prepareRequestData(Map params, boolean urlEncode)
			throws Exception {
		return utils().prepareRequestData(params, urlEncode);
	}

	/**
	 * Parses a query string of name value pairs into a Map.
	 * 
	 * @param response
	 *            A String key-value pairs in the form of a URL query string
	 * @return A Map representing the query string.
	 * @throws Exception
	 */
	public Map parseResponseData(String response) throws Exception {
		return utils().parseResponseData(response);
	}

	/**
	 * Sends an HTTP request using the POST method to a given URL.
	 * 
	 * @param urlString
	 *            The URL to send the request to.
	 * @param postData
	 *            A String representing a valid URL-encoded query string.
	 * @param followRedirect
	 *            A boolean indicating whether redirects should be followed.
	 * @param requestProperties
	 *            Map that holds request properties to be sent in connection.
	 * @param inputCharSet
	 *            Character set of the response data
	 * @return A String representing the server's response, or the Location
	 *         header if redirects are not to be followed.
	 * @throws Exception
	 */
	public String sendHTTPPost(String urlString, String postData,
			boolean followRedirect, Map requestProperties, String inputCharSet)
			throws Exception {
		return utils().sendHTTPPost(urlString, postData, followRedirect,
				requestProperties, inputCharSet);
	}

	/**
	 * Sends an HTTP request using the POST method to a given URL.
	 * 
	 * @param urlString
	 *            The URL to send the request to.
	 * @param postData
	 *            A String representing a valid URL-encoded query string.
	 * @param followRedirect
	 *            A boolean indicating whether redirects should be followed.
	 * @return A String representing the server's response, or the Location
	 *         header if redirects are not to be followed.
	 * @throws Exception
	 */
	public String sendHTTPPost(String urlString, String postData,
			boolean followRedirect) throws Exception {
		return utils().sendHTTPPost(urlString, postData, followRedirect, null,
				null);
	}

	/**
	 * Constructs a Collection of Integers composed of the productIDs of a given
	 * Collection of OrderItem objects.
	 * 
	 * @param orderItems
	 * @return A Collection of Integers representing the productIDs of the order
	 *         items.
	 * @throws Exception
	 */
	public Collection getProductIDs(Collection orderItems) throws Exception {
		Collection productIDs = new ArrayList();
		Iterator ois = orderItems.iterator();
		while (ois.hasNext()) {
			OrderItem orderItem = (OrderItem) ois.next();
			if (orderItem.getIsActive()) {
				productIDs.add(new Integer(orderItem.getProductID()));
			}
		}
		return productIDs;
	}

	/**
	 * Constructs a Collection of Integers composed of the attributeIDs of the
	 * OrderItemAttriubtes associated with a given Collection of OrderItem
	 * objects.
	 * 
	 * @param orderItems
	 * @return A Collection of Integers representing the attributeIDs of the
	 *         order items.
	 * @throws Exception
	 */
	public Collection getAttributeIDs(Collection orderItems) throws Exception {
		Collection attributeIDs = new ArrayList();
		Iterator ois = orderItems.iterator();
		while (ois.hasNext()) {
			OrderItem orderItem = (OrderItem) ois.next();
			if (orderItem.getOrderItemAttributes() != null) {
				Iterator oias = orderItem.getOrderItemAttributes().iterator();
				while (oias.hasNext()) {
					OrderItemAttribute oia = (OrderItemAttribute) oias.next();
					attributeIDs.add(new Integer(oia.getAttributeID()));
				}
			}
		}
		return attributeIDs;
	}

	/**
	 * Given a Collection of productIDs and OrderItem objects, construct a
	 * Collection of Maps composed of all of the SKUs that match the order
	 * items. Used by inventory and discount processing to handle inventory and
	 * discounts particular to given SKUs.
	 * 
	 * @param orderItems
	 * @param productIDs
	 * @return A Collection of Maps, each representing a SKU that matches one of
	 *         the order items.
	 * @throws Exception
	 */
	public Collection loadMatchingSkus(Collection orderItems,
			Collection productIDs, Collection attributeIDs) throws Exception {
		if (log.isDebugEnabled())
			log.debug("Loading matching skus for: " + productIDs);
		ProductGatewayDAO productGatewayDAO =  getDaoFactory()
				.createDAO(ProductGatewayDAO.class);
		Collection skus = productGatewayDAO.loadSkus(productIDs, attributeIDs);
		Collection skuMaps = utils().getMapsFromBeans(skus);
		ArrayList matchingSkus = new ArrayList();
		Iterator i = orderItems.iterator();
		while (i.hasNext()) {
			OrderItem orderItem = (OrderItem) i.next();
			Collection oias = orderItem.getOrderItemAttributes();
			Iterator j = skuMaps.iterator();
			while (j.hasNext()) {
				Map sku = (Map) j.next();
				// Skip if it not the same product as the order item
				if (sku.get("productID") != null
						&& !Integer.toString(orderItem.getProductID()).equals(
								sku.get("productID"))) {
					if (log.isDebugEnabled())
						log.debug("SKU does not match order item.");
					continue;
				}
				boolean match = true;
				for (int m = 1; m <= 5; m++) {
					String skuAttributeID = "0";
					String skuOptionID = "0";
					if (sku.get("attributeID" + m) != null
							&& !((String) sku.get("attributeID" + m))
									.equals("")) {
						skuAttributeID = (String) sku.get("attributeID" + m);
					}
					if (sku.get("optionID" + m) != null
							&& !((String) sku.get("optionID" + m)).equals("")) {
						skuOptionID = (String) sku.get("optionID" + m);
					}
					if (skuAttributeID.equals("0") && skuOptionID.equals("0")) {
						continue;
					}
					match = false;
					if (oias != null) {
						Iterator k = oias.iterator();
						while (k.hasNext()) {
							OrderItemAttribute oia = (OrderItemAttribute) k
									.next();
							String oiaAttributeID = Integer.toString(oia
									.getAttributeID());
							String oiaOptionID = Integer.toString(oia
									.getOptionID());
							log.debug("skuAttributeID: " + skuAttributeID);
							log.debug("skuOptionID: " + skuOptionID);
							log.debug("oiaAttributeID: " + oiaAttributeID);
							log.debug("oiaOptionID: " + oiaOptionID);
							if (oiaAttributeID.equals(skuAttributeID)
									&& oiaOptionID.equals(skuOptionID)) {
								log.debug("match!");
								match = true;
								break;
							}
						}
					}
					if (!match) {
						if (log.isDebugEnabled())
							log.debug("SKU does not match: " + sku.get("skuID")
									+ "-" + sku.get("skuNumber"));
						break;
					}
				}
				if (match) {
					if (log.isDebugEnabled())
						log.debug("Found matching sku: " + sku.get("skuID")
								+ "-" + sku.get("skuNumber"));
					Map skuToAdd = new HashMap();
					skuToAdd.putAll(sku);
					skuToAdd.put("orderItem", orderItem);
					// Adding the sku at the beginning of the array ensures they
					// will be ordered from greatest specificity to least.
					matchingSkus.add(0, skuToAdd);
				}
			}
		}
		return matchingSkus;
	}

	public void updateOrderTotals() throws Exception {
		updateOrderTotals(true);
	}

	public void updateOrderTotals(boolean subtractGiftCerts) throws Exception {
		updateOrderTotals(getUser().getOrder(), subtractGiftCerts);
	}

	/**
	 * Updates the totals of a user's order taking into account taxes and
	 * shipping. The user's <code>Order</code> and that order's first
	 * <code>OrderDelivery</code> are updated with new price totals and
	 * subtotals.
	 * 
	 * @throws Exception
	 */
	public void updateOrderTotals(Order order, boolean subtractGiftCerts) throws Exception {
		// Collect the value of all gift certificates attached to the order
		double giftCertTotal = 0;
		if (subtractGiftCerts && order.getOrderDiscounts() != null) {
			Collection orderDiscounts = order.getOrderDiscounts();
			Iterator i = orderDiscounts.iterator();
			while (i.hasNext()) {
				OrderDiscount od = (OrderDiscount) i.next();
				if (od.getItemsAppliesType() != null && od.getItemsAppliesType().equals("[giftCertificate]")) {
					giftCertTotal = giftCertTotal + od.getTotal().doubleValue();
				}
			}
		}
		// Initialize totals properties
		Double dShip = order.firstDelivery().getShipping();
		Double dTaxable = order.firstDelivery().getTaxableSubtotal();
		Double dTax = order.firstDelivery().getTax();
		Double dSub = order.firstDelivery().getSubtotal();
		if (dShip == null) {
			dShip = new Double(0);
		}
		if (dTaxable == null) {
			dTaxable = new Double(0);
		}
		if (dTax == null) {
			dTax = new Double(0);
		}
		if (dSub == null) {
			dSub = new Double(0);
		}
		// Set the order's tax and shipping based on the delivery's tax and
		// shipping
		order.setShipping(dShip);
		order.setTaxableSubtotal(dTaxable);
		order.setTax(dTax);
		Double oShip = order.getShipping();
		Double oTax = order.getTax();
		Double oSub = order.getSubtotal();
		if (oShip == null) {
			oShip = new Double(0);
		}
		if (oTax == null) {
			oTax = new Double(0);
		}
		if (oSub == null) {
			oSub = new Double(0);
		}
		// Set order total and order delivery total
		double newDeliveryTotal = dSub.doubleValue() + dTax.doubleValue() + dShip.doubleValue() - giftCertTotal;
		if (log.isDebugEnabled())
			log.debug("New delivery total: " + newDeliveryTotal);
		newDeliveryTotal = utils().roundForMoney(newDeliveryTotal).doubleValue();
		order.firstDelivery().setTotal(new Double(newDeliveryTotal));
		if (log.isDebugEnabled())
			log.debug("Delivery total: " + order.firstDelivery().getTotal());
		double newOrderTotal = oSub.doubleValue() + oTax.doubleValue() + oShip.doubleValue() - giftCertTotal;
		if (log.isDebugEnabled())
			log.debug("New order total: " + newOrderTotal);
		newOrderTotal = utils().roundForMoney(newOrderTotal).doubleValue();
		order.setTotal(new Double(newOrderTotal));
		if (log.isDebugEnabled())
			log.debug("Order total: " + order.getTotal());
		order.firstDelivery().addFormatting(getUser().getLocale(), (String) getSettings().getValue("currencyCode"));
		order.addFormatting(getUser().getLocale(), (String) getSettings().getValue("currencyCode"));
	}

	public boolean isProcessorClassActive(String componentsKey) {
		boolean ret = false;
		String[] components = getAppComponents().getProperty(componentsKey).split(",");
		for (int i = 0; i < components.length; i++) {
			String componentValues = components[i].trim();
			try {
				Class componentClass = Class.forName(componentValues);
				if (getClass().equals(componentClass)) {
					ret = true;
				}
			} catch (Exception e) {
				log.warn("Error finding if class is an active procesor in " + componentsKey + ": ", e);
			}
		}
		return ret;
		
	}
}
