package com.softslate.commerce.businessobjects.payment;

import com.softslate.commerce.businessobjects.order.Order;
import com.softslate.commerce.daos.payment.PaymentDAO;
import java.util.HashMap;
import java.util.LinkedHashSet;
import java.util.Map;
import net.authorize.ResponseField;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * Business object for AuthorizeNet Direct Post Method
 * 
 * @author Tony Zakula
 */

public class AuthorizeNetDPMProcessor extends BasePaymentProcessor implements
		PaymentProcessor {

	static Log log = LogFactory.getLog(AuthorizeNetDPMProcessor.class);

	/**
	 * This method is a assuming the action calling it has determined we have an
	 * approved transaction, and it is being used by the relay response action
	 * from Authorize.
	 */
	public Map process(Map parameters) throws Exception {

		Map result = new HashMap();
		// initially set for non-order error.
		result.put("resultCode", "1");
		// See if we have a user session or not. Depending on
		// options, we may be only inserting a payment here.
		String completeOrder = (String) getSettings().getValue(
				"authorizeNetDPMDelayOrderProcessing");

		// Here we are checking to match up the order coming back
		String orderId = "";
		try {
			// We get the invoice coming back as that is what is used throughout
			// the store
			orderId = (String) parameters.get(ResponseField.INVOICE_NUMBER
					.getFieldName());
		} catch (Exception ex) {
			if (log.isErrorEnabled())
				log.error("AuthorizeDPM transaction INVOICE NUMBER is not valid");
		}

		// If we are only inserting a payment, do not do error checking
		if (completeOrder.equals("0")) {
			if (Integer.parseInt(orderId) < 1) {
				result.put("resultCode", "2");
				return result;
			}
		} else {
			if (Integer.parseInt(orderId) < 1
					|| Integer.parseInt(orderId) != getUser().getOrder()
							.getOrderID()) {
				result.put("resultCode", "2");
				if (log.isErrorEnabled())
					log.error("Cannot match up orderID to payment coming back.");
				return result;
			}
			// Check for a possibly completed order
			if (getUser().getOrder().getCompleted() != null) {
				result.put("resultCode", "2");
				if (log.isErrorEnabled())
					log.error("Order already completed. Could be a double payment,"
							+ "but could just be a page refresh.");
				return result;
			}

			// Check for a possibly different total for a hack
			// Check parameter Map
			String total = (String) parameters.get(ResponseField.AMOUNT
					.getFieldName());
			if (total != null
					&& !(new Double(total)).equals(getUser().getOrder()
							.getTotal())) {
				if (log.isErrorEnabled())
					log.error("Cart totals and Authorize totals do not match!  "
							+ "May be a hack attempt. ");
				// record payment anyway.
			}
		} // End complete order check

		try {
			// Add the payment
			addPayment(getUser().getOrder(), parameters,
					Integer.parseInt(orderId), false);
		} catch (Exception e) {
			if (log.isErrorEnabled())
				log.error("Error handling AuthorizeDPM payment insert: ", e);
			result.put("resultCode", "2");
			return result;
		}

		// Set the resultCode to 0 to tell the callback action
		// to complete the order
		result.put("resultCode", "0");
		return result;
	}

	protected void addPayment(Order order, Map parameters, int orderId,
			boolean error) throws Exception {
		// Set payment
		Payment payment = (Payment) getBusinessObjectFactory().createObject(
				"paymentImplementer");
		String sPayType = (String) parameters.get(ResponseField.CARD_TYPE
				.getFieldName());
		if (sPayType != null && sPayType.length() > 0)
			payment.setCreditCardType(sPayType);
		payment.setCreditCardNumberDisplay("XXXX");
		// Collect a string repesentation for all parameters
		int mapsize = parameters.size();
		String textValues = "";
		String ls = System.getProperty("line.separator");
		Object[] keyValuePairs = parameters.entrySet().toArray();
		for (int i = 0; i < mapsize; i++) {
			Map.Entry entry = (Map.Entry) keyValuePairs[i];
			textValues += entry.getKey() + "=" + entry.getValue() + ls;
		}
		if (log.isDebugEnabled())
			log.debug(textValues);

		payment.setProcessorClassName(this.getClass().getName());
		payment.setCreated(formatDateTime(new java.util.Date()));
		try {
			String amountS = (String) parameters.get(ResponseField.AMOUNT
					.getFieldName());
			Double amountD = new Double(amountS);
			payment.setAmount(amountD);
		} catch (Exception e) {
			if (log.isWarnEnabled()) {
				log.warn("Error parsing payment amount: ", e);
			}
		}

		// We pass a boolean just in case we want to record error payments
		// in the future.
		if (error) {
			payment.setStatus("Error");
		} else {
			payment.setStatus("Received");
		}

		payment.setValueFieldLabels("TRANSACTION_TYPE,AMOUNT,CARD_CODE,RESPONSE_CODE,AVS_CODE,AUTHCODE,TRANSACTION_ID,REASON_CODE,REASON_TEXT");
		payment.setValue1((String) parameters.get(ResponseField.METHOD
				.getFieldName()));
		payment.setValue2((String) parameters.get(ResponseField.AMOUNT
				.getFieldName()));
		// payment.setValue3((String) response.get(RESPONSE_CVV2_CODE));
		payment.setValue4((String) parameters.get(ResponseField.RESPONSE_CODE
				.getFieldName()));
		payment.setValue5((String) parameters.get(ResponseField.AVS_CODE
				.getFieldName()));
		payment.setValue6((String) parameters
				.get(ResponseField.AUTHORIZATION_CODE.getFieldName()));
		payment.setValue7((String) parameters.get(ResponseField.TRANSACTION_ID
				.getFieldName()));
		payment.setValue8((String) parameters.get(ResponseField.RESPONSE_CODE
				.getFieldName()));
		payment.setValue9((String) parameters
				.get(ResponseField.RESPONSE_REASON_TEXT.getFieldName()));
		payment.setOrder(order);
		if (order.getPayments() == null)
			order.setPayments(new LinkedHashSet());
		order.getPayments().add(payment);
		PaymentDAO paymentDAO = (PaymentDAO) getDaoFactory().createDAO(
				"paymentDAOImplementer");
		paymentDAO.setPayment(payment);
		paymentDAO.insertPayment(false);

	}

}
