package com.softslate.commerce.businessobjects.payment;

import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.softslate.commerce.businessobjects.core.BaseBusinessProcessor;
import com.softslate.commerce.businessobjects.order.Order;
import com.softslate.commerce.daos.payment.PaymentDAO;

/**
 * Abstract class that provides methods useful for <code>PaymentProcessor</code>
 * implementations.
 * 
 * @author David Tobey
 */
public abstract class BasePaymentProcessor extends BaseBusinessProcessor
		implements PaymentProcessor {

	static Log log = LogFactory.getLog(BasePaymentProcessor.class);

	/**
	 * Subclasses must override this method to execute processing. Called by
	 * {@link #processPayment(Map)}.
	 * 
	 * @param parameters
	 *            A <code>Map</code> containing the values of the user's
	 *            submitted payment information.
	 * @return A <code>Map</code> containing a least one key named
	 *         "resultCode", which contains a String parsable into an int,
	 *         representing the results of the processing. 0 for the resultCode
	 *         indicates success. Optionally, the <code>Map</code> may include
	 *         additional keys, such as "resultMessage", a string describing the
	 *         results of the processing.
	 * @throws Exception
	 */
	public abstract Map process(Map parameters) throws Exception;

	/**
	 * Assists with processing of payments. Copies the incoming <code>Map</code>
	 * of parameters to this <code>BasePaymentProcessor</code> instance and
	 * calls <code>process()</code>. Catches any exception that occur within
	 * the processor and marks the user's order as corrupt if any occur. This
	 * method also skips processing if the store's "paymentRequired" settings is
	 * set to false.
	 * 
	 * @param parameters
	 *            A <code>Map</code> containing the values of the user's
	 *            submitted payment information.
	 * @return A <code>Map</code> containing a least one key named
	 *         "resultCode", which contains a String parsable into an int,
	 *         representing the results of the processing. 0 for the resultCode
	 *         indicates success. Optionally, the <code>Map</code> may include
	 *         additional keys, such as "resultMessage", a string describing the
	 *         results of the processing.
	 */
	public Map processPayment(Map parameters) throws Exception {
		if (log.isInfoEnabled())
			log.info("Processing payment.");
		Map result = new HashMap();
		result.put("resultCode", "0");
		try {
			// Copy the incoming parameters to this object.
			String paymentRequired = (String) getSettings().getValue(
					"paymentRequired");
			if (paymentRequired.equals("1")) {
				if (log.isDebugEnabled())
					log.debug("Processing payment.");

				result = process(parameters);
			}
		} catch (Exception e) {
			// Set the Order as corrupted if something bad happened.
			if (getUser().getOrder().getCompleted() == null) {
				getUser().getOrder().setStatus("Corrupted");
			}
			throw e;
		}
		return result;
	}

	/**
	 * A utility method that returns the first Payment under the given Order
	 * that has a full credit card number stored. It first decrypts the credit
	 * card number (if called for by the settings). Useful for functions that
	 * must look up the credit card information for a given order.
	 * 
	 * @param order
	 *            The Order being examined.
	 * @return A Payment object that has a credit card number stored with it.
	 * @throws Exception
	 */
	public Payment findPaymentWithLiveInfo(Order order) throws Exception {
		if (order == null) {
			return null;
		}
		PaymentDAO paymentDAO = (PaymentDAO) getDaoFactory().createDAO(
				"paymentDAOImplementer");
		Collection payments = order.getPayments();
		if (payments != null && !payments.isEmpty()) {
			Iterator i = payments.iterator();

			// Loop through the order's payments and find the first one with a
			// credit card number.
			while (i.hasNext()) {
				Payment p = (Payment) i.next();
				// Set the decrypted number so we can use later
				if (p.getCreditCardNumber() != null) {
					String paymentEncryptionType = (String) getSettings()
							.getValue("paymentEncryptionType");
					p.setDecryptedNumber(paymentDAO.getDAOUtils().getDecrypted(
							p.getCreditCardNumber(), paymentEncryptionType,
							getAppSettings().getProperty("twoWayKeyFile")));
					if (paymentEncryptionType.equals("noEncryption")
							|| !p.getCreditCardNumber().equals(
									p.getDecryptedNumber())) {
						return p;
					}
				}
			}
		}
		return null;
	}

	public Map processPaymentCapture(Order order) throws Exception {
		log.info("processPaymentCapture not supported for payment processor: " + this.getClass().getName());
		return null;
	}

	public Map processVoid(Map parameters) throws Exception {
		log.info("processPaymentCapture not supported for payment processor: " + this.getClass().getName());
		return null;
	}
}
