package com.softslate.commerce.businessobjects.payment;

import java.util.Map;

import com.softslate.commerce.businessobjects.core.BusinessProcessor;
import com.softslate.commerce.businessobjects.order.Order;

/**
 * Interface for objects that are able to process payments in the store. Custom
 * payment processors must simply implement <code>PaymentProcessor</code>.
 * (Although it may be useful also to extend
 * {@link com.softslate.commerce.businessobjects.payment.BasePaymentProcessor}.)
 * <p>
 * To change or add to the list of payment processors currently active in the
 * system, modify the "activePaymentProcessors" setting in the
 * <code>appComponents.properties</code> file.
 * <p>
 * During checkout the Struts layer loops through each of the
 * "activePaymentFormTemplates" to display a set of payment forms for the user.
 * Based on the form that user chooses to submit, the Struts layer calls the
 * "activePaymentProcessor" that corresponds to it. It invokes the
 * {@link #processPayment(Map)} method to process the submission. The incoming
 * <code>Map</code> contains the various input parameters submitted by the
 * user.
 * <p>
 * There is one default "activePaymentProcessor":
 * {@link com.softslate.commerce.businessobjects.payment.BasicPaymentProcessor}.
 * 
 * @author David Tobey
 * 
 */
public interface PaymentProcessor extends BusinessProcessor {
	/**
	 * Process the payment for the current user's order. The incoming
	 * <code>Map</code> contains the various input parameters submitted by the
	 * user.
	 * 
	 * @param parameters
	 *            A <code>Map</code> containing the values of the user's
	 *            submitted payment information.
	 * @return A <code>Map</code> containing a least one key named
	 *         "resultCode", which contains a String parsable into an int,
	 *         representing the results of the processing. 0 for the resultCode
	 *         indicates success. Optionally, the <code>Map</code> may include
	 *         additional keys, such as "resultMessage", a string describing the
	 *         results of the processing.
	 * @throws Exception
	 */
	abstract Map processPayment(Map parameters) throws Exception;

	abstract Map processPaymentCapture(Order order) throws Exception;

	abstract Map processVoid(Map parameters) throws Exception;

}
