package com.softslate.commerce.businessobjects.payment.cresecure;

import java.util.Map;

import com.softslate.commerce.businessobjects.customer.Customer;
import com.softslate.commerce.businessobjects.customer.CustomerAddress;
import com.softslate.commerce.businessobjects.order.Order;
import com.softslate.commerce.businessobjects.payment.PaymentProcessor;
import com.softslate.commerce.customer.payment.cresecure.CRESecureCompleteForm;

/**
 * Interface that handles CRE Secure Communication
 * 
 * @author Jason McWilliams
 */

public interface CRESecureProcessor extends PaymentProcessor {

	/**
	 * Processes the payment using the form sent in. Use this instead of process
	 * by sending in a map.
	 * 
	 * @param form
	 * @return
	 * @throws Exception
	 */
	abstract Map process(CRESecureCompleteForm form) throws Exception;

	/**
	 * Processes a payment using an existing token
	 * 
	 * @param parameters
	 * @return
	 * @throws Exception
	 */
	abstract Map process(Map parameters) throws Exception;


	abstract void fillCustomerAddressFromPayment(CustomerAddress address, CRESecureCompleteForm form);

	/**
	 * Prepares the frame form URL. This may be a string of all the data or a
	 * special uID depending on the setting.
	 * 
	 * @param order
	 * @param customer
	 * @param sessionId
	 * @return
	 * @throws Exception
	 */
	abstract String prepareFormURL(Order order, Customer customer, String sessionId) throws Exception;

	/**
	 * Get the transactions results from uid
	 * 
	 * @param uID
	 * @return
	 */
	abstract Map requestTransactionResults(String uID);

	/**
	 * Use the new authorization direct service to request a new authorization
	 * with a card token. Once you have a card token from CRE Secure you can
	 * send the token via a web service call to request new authorizations
	 * (charges) against the card that the card token represents. Send an HTTP
	 * POST to the following endpoint:
	 * https://direct.cresecure.net/direct/services/authorize
	 * 
	 * @return
	 */
	abstract CRESecureAuthorizationResult chargeNewAuthorization(Order order, Customer customer, String ipAddress)
			throws Exception;

	/**
	 * The transaction validation service allows you to query the CRE Secure
	 * platform to verify that a payment was successfully processed. This
	 * enables you to automate verification of orders marked as paid to prevent
	 * payment spoofing. To validate a transaction, send an HTTP POST to the
	 * following endpoint:
	 * https://direct.cresecure.net/direct/services/validation
	 * 
	 * @return
	 */
	abstract CRESecureValidationResult validateTransaction(Order order, String transactionID) throws Exception;

	/**
	 * Note: The reversal feature will be included in a future release. Use the
	 * Transaction Reversal Direct Service to void or refund a transaction. Send
	 * an HTTP GET to perform a transaction reversal to the following endpoint:
	 * https://direct.cresecure.net/direct/services/reversal
	 * 
	 * @return
	 */
	abstract Map transactionReversal();

	/**
	 * This will update the customer addresses in the database after back
	 * filling the CRE response information into the address book entry. The
	 * address sent in should contain update customer
	 * 
	 * @param address
	 * @param form
	 * @return
	 * @throws Exception
	 */
	abstract Map updateCustomerAddressBook(int customerAddressID, CRESecureCompleteForm form) throws Exception;
}
