package com.softslate.commerce.businessobjects.payment.cresecure;

import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.LinkedHashSet;
import java.util.Map;
import java.util.StringTokenizer;

import org.apache.commons.beanutils.BeanUtils;
import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.google.gson.Gson;
import com.google.gson.GsonBuilder;
import com.softslate.commerce.businessobjects.core.ApplicationException;
import com.softslate.commerce.businessobjects.customer.Customer;
import com.softslate.commerce.businessobjects.customer.CustomerAddress;
import com.softslate.commerce.businessobjects.order.Order;
import com.softslate.commerce.businessobjects.payment.BasePaymentProcessor;
import com.softslate.commerce.businessobjects.payment.Payment;
import com.softslate.commerce.businessobjects.payment.PaymentProcessor;
import com.softslate.commerce.businessobjects.payment.authorizeNetCIM.AuthorizeNetCIMProcessor;
import com.softslate.commerce.businessobjects.payment.authorizeNetCIM.AuthorizeNetCIMResult;
import com.softslate.commerce.customer.payment.cresecure.CRESecureCompleteForm;
import com.softslate.commerce.daos.customer.CustomerGatewayDAO;
import com.softslate.commerce.daos.payment.PaymentDAO;

/**
 * Interface that handles CRE Secure Communication
 * 
 * @author Jason McWilliams
 */
public class CRESecureProcessorImpl extends BasePaymentProcessor implements PaymentProcessor, CRESecureProcessor {

	static Log log = LogFactory.getLog(CRESecureProcessorImpl.class);

	static final String VALUE_FIELDS = "transId,appCode,CVVMatch,AVSMatch,approvalCode,status,trans_type,tokenId,jsonResult";

	public static final Map<String, String> RESULT_CODES = Collections.unmodifiableMap(new HashMap<String, String>() {
		/**
		 * Default messages if the from the processor in case they need to be
		 * looked up
		 */
		private static final long serialVersionUID = -873849923898961328L;

		{
			put("100",
					"Merchant Identifier left blank or not valid. Indicates a corrupt or hacked payment form. The transaction was not processed.");
			put("110",
					"Session Identifier left blank. Indicates a corrupt or hacked payment form. The transaction was not processed.");
			put("200", "Name not present.");
			put("500", "Address one field required but left blank.");
			put("510", "City field required but left blank.");
			put("520", "State field required but left blank.");
			put("530", "ZIP/postal code field required but left blank.");
			put("531", "ZIP/postal code field is invalid.");
			put("300", "Amount not specified or invalid value entered");
			put("310", "Credit card number left blank.");
			put("315", "Credit card number did not pass Mod10.");
			put("320", "Invalid card brand");
			put("330", "Expiration month left blank.");
			put("340", "Expiration year left blank.");
			put("350", "CVV2 field submitted but does not match the card.");
			put("355", "CVV2 field required, but left blank.");
			put("357", "CVV2 field is invalid.");
			put("360", "Card declined by the processor.");
			put("370", "Expiration date invalid.");
			put("400", "Tracer value does not match.");
			put("600", "The Bank name was blank (ACH Only)");
			put("610", "The Routing Number is blank (ACH Only)");
			put("620", "The Checking Account number was blank (ACH Only)");
			put("640", "The Routing Number is invalid (ACH Only)");
			put("700", "Fraudulent Order");
		}
	});

	public Map process(CRESecureCompleteForm form) throws Exception {
		log.info("Processing after cre form payment.");

		if (log.isInfoEnabled())
			log.info("Processing payment.");
		Payment payment = (Payment) getBusinessObjectFactory().createObject("paymentImplementer");

		payment.setStatus("Received");
		String displayCheckoutConfirm = (String) getSettings().getValue("displayCheckoutConfirm");
		if ("1".equals(displayCheckoutConfirm)) {
			payment.setAmount(0.00);
		} else {
			payment.setAmount(getUser().getOrder().getTotal());
		}
		payment.setCreated(formatDateTime(new java.util.Date()));
		payment.setOrder(getUser().getOrder());
		payment.setProcessorClassName(this.getClass().getName());
		payment.setCreditCardNumberDisplay(form.getCcNumber());
		String creditCardDate = StringUtils.stripToEmpty(form.getExpYear())
				+ StringUtils.stripToEmpty(form.getExpMonth());
		payment.setCreditCardDate(creditCardDate);
		payment.setCreditCardName(form.getName());
		payment.setCreditCardType(form.getCcType());
		// CRE May return this which we can store from CIM if they are using CIM
		payment.setPaymentProfileID(form.getCustomerPaymentProfileId());
		payment.setValue1(form.getTransId());
		payment.setValue2(form.getAppCode());
		payment.setValue3(form.getCVVMatch());
		payment.setValue4(form.getAVSMatch());
		payment.setValue5(form.getApprovalCode());
		payment.setValue6(form.getStatus());
		if ("1".equals(displayCheckoutConfirm)) {
			payment.setValue7("store_only");
		} else {
			payment.setValue7((String) getSettings().getValue("creSecureTransType"));
		}

		// This is the CRE Token we can use to make payments later or we could
		// use payment profile if using CIM
		payment.setValue8(form.getTokenId());

		// Store the entire result in json that can be looked at later if needed
		String allFields = null;
		try {
			Gson gson = new GsonBuilder().excludeFieldsWithoutExposeAnnotation().create();
			allFields = gson.toJson(form);
		} catch (Exception ex) {
			if (log.isErrorEnabled()) {
				log.error("Exception creating cre json payment data for storage.", ex);
			}
			allFields = null;
		}
		payment.setValue9(allFields);

		payment.setValueFieldLabels(VALUE_FIELDS);

		payment.setOrder(getUser().getOrder());
		if (getUser().getOrder().getPayments() == null) {
			getUser().getOrder().setPayments(new LinkedHashSet());
		}
		getUser().getOrder().getPayments().add(payment);
		PaymentDAO paymentDAO = (PaymentDAO) getDaoFactory().createDAO("paymentDAOImplementer");
		paymentDAO.setPayment(payment);
		// Don't commit the transaction, so clean up queries can follow.
		paymentDAO.insertPayment(false);
		Map result = new HashMap();
		result.put("resultCode", "0");
		return result;
	}

	public Map process(Map parameters) throws Exception {
		log.info("Processing cre token payment.");
		Map result = new HashMap();

		Order order = getUser().getOrder();
		Customer customer = getUser().getCustomer();
		String token = null;
		if (order != null) {
			token = order.getPaymentProfileID();
		}

		CustomerAddress foundAddress = null;
		if (StringUtils.isNotBlank(token) && customer != null && customer.getCustomerAddressCollection() != null) {
			for (CustomerAddress addr : (Collection<CustomerAddress>) getUser().getCustomer()
					.getCustomerAddressCollection()) {
				if (StringUtils.isNotBlank(addr.getPaymentToken()) && token.equals(addr.getPaymentToken())) {
					foundAddress = addr;
					break;
				}
			}
		}

		CRESecureAuthorizationResult authResult = chargeNewAuthorization(getUser().getOrder(), getUser().getCustomer(),
				(String) parameters.get("ipAddress"));
		if (authResult != null && "000".equals(authResult.getCode())) {
			Payment payment =  getBusinessObjectFactory().createObject(Payment.class);

			payment.setStatus("Received");
			payment.setAmount(getUser().getOrder().getTotal());
			payment.setCreated(formatDateTime(new java.util.Date()));
			payment.setOrder(getUser().getOrder());
			payment.setProcessorClassName(this.getClass().getName());
			if (foundAddress != null) {
				payment.setCreditCardNumberDisplay(foundAddress.getCreditCardNumberDisplay());
				payment.setCreditCardDate(foundAddress.getCreditCardDate());
				payment.setCreditCardName(foundAddress.getCreditCardName());
				payment.setCreditCardType(foundAddress.getCreditCardType());
			}
			// Set these if we get a result with the undocumented fields instead
			if (StringUtils.isNotBlank(authResult.getMpan())) {
				payment.setCreditCardNumberDisplay(authResult.getMpan());
			}
			if (StringUtils.isNotBlank(authResult.getExp())) {
				payment.setCreditCardDate(authResult.getExp());
			}
			if (StringUtils.isNotBlank(authResult.getName())) {
				payment.setCreditCardName(authResult.getName());
			}
			if (StringUtils.isNotBlank(authResult.getType())) {
				payment.setCreditCardType(authResult.getType());
			}
			payment.setPaymentProfileID(null);
			payment.setValue1(authResult.getTxnguid());
			payment.setValue2(null);
			payment.setValue3(null);
			payment.setValue4(null);
			payment.setValue5(authResult.getApprovalcode());
			payment.setValue6(authResult.getCode());
			// TODO: It never makes it clear what type of auth this is and its
			// not clear we can tell it
			payment.setValue7((String) getSettings().getValue("creSecureTransType"));
			payment.setValue8(getUser().getOrder().getPaymentProfileID());
			String allFields = null;
			try {
				Gson gson = new GsonBuilder().create();
				allFields = gson.toJson(authResult);
			} catch (Exception ex) {
				if (log.isErrorEnabled()) {
					log.error("Exception creating cre json payment data for storage.", ex);
				}
				allFields = null;
			}
			payment.setValue9(allFields);
			payment.setValueFieldLabels(VALUE_FIELDS);
			payment.setOrder(getUser().getOrder());
			if (getUser().getOrder().getPayments() == null) {
				getUser().getOrder().setPayments(new LinkedHashSet());
			}
			getUser().getOrder().getPayments().add(payment);
			PaymentDAO paymentDAO =  getDaoFactory().createDAO(PaymentDAO.class);
			paymentDAO.setPayment(payment);
			// Don't commit the transaction, so clean up queries can follow.
			paymentDAO.insertPayment(false);
			result.put("resultCode", "0");
		} else {
			result.put("resultCode", authResult.getCode());
			result.put("resultMessage", authResult.getMessage());
			result.put("resultErrorMessage", authResult.getError());
		}

		return result;
	}

	public void fillCustomerAddressFromPayment(CustomerAddress address, CRESecureCompleteForm form) {
		if (form == null || address == null) {
			return;
		}
		address.setIsBilling(true);
		if (StringUtils.isNotBlank(form.getName())) {
			String[] names = form.getName().trim().split("\\s+", 2);
			if (names.length > 0) {
				address.setFirstName(names[0]);
			}
			if (names.length > 1) {
				address.setLastName(names[1]);
			}
		}
		if (StringUtils.isNotBlank(form.getAddress1())) {
			address.setAddress1(form.getAddress1());
			address.setAddress2(form.getAddress2());
		}
		if (StringUtils.isNotBlank(form.getCity())) {
			address.setCity(form.getCity());
		}
		if (StringUtils.isNotBlank(form.getProvince())) {
			address.setState(form.getProvince());
		}
		if (StringUtils.isNotBlank(form.getPostal_code())) {
			address.setPostalCode(form.getPostal_code());
		}
		if (StringUtils.isNotBlank(form.getCountry())) {
			address.setCountry(form.getCountry());
		}
		if (StringUtils.isNotBlank(form.getTokenId()) && StringUtils.isBlank(address.getPaymentToken())) {
			address.setPaymentToken(form.getTokenId());
		}
		address.setCreditCardDate(StringUtils.stripToEmpty(form.getExpYear())
				+ StringUtils.stripToEmpty(form.getExpMonth()));
		address.setCreditCardName(StringUtils.stripToEmpty(form.getName()));
		address.setCreditCardNumberDisplay(StringUtils.stripToEmpty(form.getCcNumber()));
		address.setCreditCardType(StringUtils.stripToEmpty(form.getCcType()));
	}

	public String prepareFormURL(Order order, Customer customer, String sessionId) throws Exception {
		if (order == null) {
			return null;
		}

		// If using confirm, only add the card, do not charge anything.
		String displayCheckoutConfirm = (String) getSettings().getValue("displayCheckoutConfirm");

		Map<String, String> params = new HashMap<String, String>();
		params.put("collectAddress", (String) getSettings().getValue("creSecureCollectAddress"));
		params.put("action", "buildForm");
		params.put("merchPass", (String) getSettings().getValue("creSecureMerchPass"));
		params.put("trans_type", (String) getSettings().getValue("creSecureTransType"));

		if ("1".equals(displayCheckoutConfirm)) {
			params.put("cre_tokenize", "store_only");
		} else {
			params.put("cre_tokenize", (String) getSettings().getValue("creSecureTokenize"));
		}

		params.put("allowed_types", (String) getSettings().getValue("creSecureAllowedTypes"));
		params.put("formType", (String) getSettings().getValue("creSecureFormType"));
		params.put("paymentType", "");
		params.put("address", StringUtils.stripToEmpty(order.getAddress1()));
		params.put("address2", StringUtils.stripToEmpty(order.getAddress2()));
		params.put("city", StringUtils.stripToEmpty(order.getCity()));
		params.put("state", StringUtils.stripToEmpty(order.getState()));
		params.put("zip", StringUtils.stripToEmpty(order.getPostalCode()));

		int customerID = 0;
		if (customer != null && customer.getCustomerID() > 0) {
			customerID = customer.getCustomerID();
			params.put("customer_id", Integer.toString(customerID));
		}
		// params.put("order_desc", "");

		params.put("customer_email", StringUtils.stripToEmpty(order.getEmail1()));

		// TODO: Find out exactly what this does
		// params.put("analysis", "");

		params.put("required", (String) getSettings().getValue("creSecureRequired"));
		String name = "";
		if (StringUtils.isNotBlank(order.getFirstName())) {
			name = order.getFirstName();
		}
		if (StringUtils.isNotBlank(order.getLastName())) {
			if (StringUtils.isNotBlank(name)) {
				name = name + " " + order.getLastName();
			} else {
				name = order.getLastName();
			}

		}
		params.put("name", name);

		// Use default token stuff
		// params.put("token_type", "");
		// params.put("token_slug", "");
		// params.put("CRE_custom_token", "");
		if ("1".equals(displayCheckoutConfirm)) {
			// Don't send amount, only storing card
		} else {
			String amountStringTwoDecimals = "";
			if (order.getTotal() != null) {
				amountStringTwoDecimals = String.format("%.2f", order.getTotal());
			}
			params.put("amount", amountStringTwoDecimals);
		}

		// Only USD/US Right now
		params.put("cy_code", "USD");
		params.put("lang", "en_US");

		params.put("sessionId", sessionId);
		params.put("orderId", Integer.toString(order.getOrderID()));

		String creSecureCSSURL = (String) getSettings().getValue("creSecureCSSURL");
		if (StringUtils.isNotBlank(creSecureCSSURL)) {
			if (!creSecureCSSURL.startsWith("http")) {
				creSecureCSSURL = getSettings().getValue("customerSecureURL") + creSecureCSSURL;
			}
			params.put("css_url", creSecureCSSURL);
		}

		String queryString = prepareRequestData(params, true);
		String url = getSettings().getValue("creSecureIframeURL").toString() + "?" + queryString;

		return url;

	}

	public Map requestTransactionResults(String uID) {
		return null;
	}

	public CRESecureAuthorizationResult chargeNewAuthorization(Order order, Customer customer, String ipAddress)
			throws Exception {
		CRESecureAuthorizationResult result = new CRESecureAuthorizationResult();

		if (order == null || StringUtils.isBlank(order.getPaymentProfileID())) {
			result.setCode("999");
			result.setError("INTERNAL");
			result.setMessage("Invalid order or payment information not found.");
			return result;
		}

		Map<String, String> params = new HashMap<String, String>();

		// Required Unique Account Number for CRE Secure Service provided by CRE
		// Secure.
		params.put("CRESecureID", (String) getSettings().getValue("creSecureMerchPass"));
		// Required API Token provided by CRE Secure.
		params.put("CRESecureAPIToken", (String) getSettings().getValue("creSecureAPIToken"));
		// Optional System value of the order or payment number. 1 character
		// minimum.
		// Docs are wrong about the name of this field
		// params.put("orderId", null);
		params.put("order_id", Integer.toString(order.getOrderID()));
		// Required The transaction amount to authorize or capture. Numeric, 2
		// decimal places.
		String amountStringTwoDecimals = String.format("%.2f", order.getTotal());
		params.put("total_amt", amountStringTwoDecimals);
		// Required The card token representing the card being used for the
		// transaction.
		params.put("CRE_Card_Token", order.getPaymentProfileID());
		// Optional The customer identification number. 1 character minimum.
		int customerID = 0;
		if (customer != null && customer.getCustomerID() > 0) {
			customerID = customer.getCustomerID();
			params.put("customer_id", Integer.toString(customerID));
		}
		// Optional The billing address zip or postal code. 5 characters
		// minimum.
		if (order.getPostalCode() != null) {
			params.put("customer_zip_code", order.getPostalCode());
		}
		// Optional The customer's IP address.
		params.put("ip_address", ipAddress);
		// Optional Text description of the purchase. 1 character minimum.
		// params.put("order_desc", null);

		// Optional The total weight of the order. Determines if physical or
		// digital goods. If no value is set, default is 0.
		if (order.getWeight() != null) {
			params.put("total_weight", Double.toString(order.getWeight()));
		}
		// Optional The three-letter ISO currency code for the transaction. If
		// no value is set, default is USD.
		// params.put("currency_code", null);
		// Optional Enables the ability to send the token directly to the
		// gateway and bypass the CRE Secure servers. Only works with a
		// gateway's token. This parameter takes the following values: null -
		// Disables the pass through feature True - Enables the pass through
		// feature
		// params.put("pass_thru", null);

		try {
			String queryString = prepareRequestData(params, true);
			String creSecureAuthorizationServiceURL = (String) getSettings().getValue(
					"creSecureAuthorizationServiceURL");
			String responseData = sendHTTPPost(creSecureAuthorizationServiceURL, queryString, false);

			if (responseData != null) {
				String[] data = responseData.split("&");
				if (data != null) {
					for (int i = 0; i < data.length; i++) {
						StringTokenizer st = new StringTokenizer(data[i].trim(), "=");
						while (st.hasMoreTokens()) {
							String key = st.nextToken();
							String val = st.nextToken();
							try {
								// Go lower case because of strange docs or
								// return values?
								BeanUtils.setProperty(result, key.toLowerCase(), val);
							} catch (Exception setex) {
								// Swallow
								if (log.isErrorEnabled()) {
									log.error("Error setting bean props from result", setex);
								}
							}
						}
					}
				}
				return result;
			} else {
				result.setCode("999");
				result.setError("INTERNAL");
				result.setMessage("Unable to get response from processor.");
				return result;
			}
		} catch (Exception ex) {
			if (log.isErrorEnabled()) {
				log.error("Exception processing payment. ", ex);
			}
			result.setCode("999");
			result.setError("INTERNAL");
			result.setMessage("Exception while processing payment.");
			return result;
		}

	}

	public CRESecureValidationResult validateTransaction(Order order, String transactionID) throws Exception {
		CRESecureValidationResult result = new CRESecureValidationResult();
		// Call the verification service
		// String creSecureValidationServiceURL =
		// "https://direct.cresecure.net/direct/services/validation";
		String creSecureValidationServiceURL = (String) getSettings().getValue("creSecureValidationServiceURL");
		Map<String, String> requestData = new HashMap<String, String>();
		requestData.put("CRESecureID", (String) getSettings().getValue("creSecureMerchPass"));
		requestData.put("CRESecureAPIToken", (String) getSettings().getValue("creSecureAPIToken"));
		requestData.put("order_id", Integer.toString(order.getOrderID()));
		String amountStringTwoDecimals = String.format("%.2f", order.getTotal());
		requestData.put("total_amt", amountStringTwoDecimals);
		requestData.put("trans_id", transactionID);
		// This is ignoring return type, going to have to parse nvp
		// requestData.put("rtype", "xml");

		String postData = prepareRequestData(requestData, true);
		String responseData = sendHTTPPost(creSecureValidationServiceURL, postData, false);
		if (responseData != null) {
			try {

				// CRESecureValidationResult.class);
				String[] data = responseData.split("&");
				if (data != null) {
					for (int i = 0; i < data.length; i++) {
						StringTokenizer st = new StringTokenizer(data[i].trim(), "=");
						while (st.hasMoreTokens()) {
							String key = st.nextToken();
							String val = st.nextToken();
							try {
								// Go lower case because of strange docs or
								// return values?
								BeanUtils.setProperty(result, key.toLowerCase(), val);
							} catch (Exception setex) {
								// Swallow
								if (log.isErrorEnabled()) {
									log.error("Error setting bean props from result", setex);
								}
							}
						}
					}
				}

				if ("success".equalsIgnoreCase(result.getStatus())
						&& "YES".equalsIgnoreCase(result.getOrder_id_match())
						&& "YES".equalsIgnoreCase(result.getTotal_amt_match())) {

					result.setIsValid(true);

				} else {
					result.setIsValid(false);
					result.setError("Error validating payment.");
					if (log.isErrorEnabled()) {
						log.error("Error validating payment, null result: " + order.getOrderID());
					}
				}

			} catch (Exception ex) {
				result = new CRESecureValidationResult();
				result.setValid(false);
				result.setError("Error validating payment.");
				if (log.isErrorEnabled()) {
					log.error("Exception validating payment, null result: " + order.getOrderID(), ex);
				}
			}
		}

		return result;
	}

	public Map transactionReversal() {
		return null;
	}

	public Map updateCustomerAddressBook(int customerAddressID, CRESecureCompleteForm form) throws Exception {
		if (log.isInfoEnabled())
			log.info("Updating customer and address book.");
		if (getUser().getCustomer() == null) {
			throw new ApplicationException("User is not logged in; cannot update addresses.");
		}
		// Refresh the customer object from the database
		CustomerGatewayDAO customerGatewayDAO = (CustomerGatewayDAO) getDaoFactory().createDAO(
				"customerGatewayDAOImplementer");
		Map result = customerGatewayDAO.refreshCustomerFromDatabase(getUser());
		if (result.get("resultCode") != null && result.get("resultCode").equals("0")) {
			// Save the current customer in case we have to revert to it
			Map existing = PropertyUtils.describe(getUser().getCustomer());
			CustomerAddress foundAddress = null;
			// After refresh, we have to find the matching address
			if (getUser() != null && getUser().getCustomer() != null
					&& getUser().getCustomer().getCustomerAddressCollection() != null) {
				for (CustomerAddress addr : (Collection<CustomerAddress>) getUser().getCustomer()
						.getCustomerAddressCollection()) {
					if (customerAddressID == addr.getCustomerAddressID()) {
						foundAddress = addr;
						break;
					}
				}
			}
			if (foundAddress != null) {
				// Backfill properties
				// If the original address had a payment token and it is now
				// different, the user must have entered a new CC for an
				// existing billing address
				if (StringUtils.isNotBlank(foundAddress.getPaymentToken()) && StringUtils.isNotBlank(form.getTokenId())
						&& !foundAddress.getPaymentToken().trim().equals(form.getTokenId().trim())) {
					CustomerAddress customerAddress = (CustomerAddress) getBusinessObjectFactory().createObject(
							"customerAddressImplementer");
					BeanUtils.copyProperties(customerAddress, foundAddress);
					// Reset backfill and problematic info
					customerAddress.setCustomerAddressID(0);
					customerAddress.setPaymentToken(null);
					customerAddress.setCreditCardDate(null);
					customerAddress.setCreditCardName(null);
					customerAddress.setCreditCardNumberDisplay(null);
					customerAddress.setCreditCardType(null);
					customerAddress.setIdentifier(null);
					customerAddress.setIsDefault(false);
					customerAddress.setCustomerID(0);
					customerAddress.setCustomer(getUser().getCustomer());
					fillCustomerAddressFromPayment(customerAddress, form);
					getUser().getCustomer().getCustomerAddressCollection().add(customerAddress);
				} else {
					fillCustomerAddressFromPayment(foundAddress, form);
				}
				// Update the database
				result = customerGatewayDAO.updateAddresses(getUser().getCustomer());
				// Revert to the previous info if any errors
				if (result != null && result.get("resultCode") != null && !result.get("resultCode").equals("0")) {
					BeanUtils.copyProperties(getUser().getCustomer(), existing);
				}
			}
		}
		return null;
	}

	/**
	 * This method is called from the events system when the setting
	 * "Order Status for Capturing Payments" is set and matches the event.
	 */
	public Map processPaymentCapture(Order order) throws Exception {
		Map result = new HashMap();
		if ("1".equals(getSettings().getValue("creSecureCapturePriorAuthPaymentsViaCIM"))) {
			log.info("Processing CRESecure processPaymentCapture");
			AuthorizeNetCIMProcessor acimp = getBusinessObjectFactory().createObject(AuthorizeNetCIMProcessor.class);

			// Check to see if we have a payment for this order that is captured
			Collection<Payment> payments = order.getPayments();
			if (payments != null && payments.size() > 0) {
				log.info("Looking to see if we have a captured payment");
				for (Payment payment : payments) {
					if (payment.getValue1() != null && !payment.getValue1().isEmpty()
							&& payment.getValue1().equals("prior_auth_capture")) {
						log.info("Found payment with captured code " + payment.getValue1());
						result.put("resultCode", "-1");
						result.put("resultMessage", "Found payment with captured code " + payment.getValue1());
						return result;
					}
				}

				// Check if we have a payment with an authorization code
				log.info("Looking for payment with authorization code");
				for (Payment payment : payments) {

					// Transaction ID is in value1, Auth Code is in value2
					if (payment.getValue7() != null && !payment.getValue7().isEmpty()
							&& payment.getValue7().equals("auth_only") && payment.getValue1() != null
							&& !payment.getValue1().isEmpty() && payment.getValue1() != null
							&& !payment.getValue2().isEmpty()) {
						log.info("Found payment with transaction id " + payment.getValue1()
								+ " and authorization code " + payment.getValue2());

						// Check if the order total is greater and throw an
						// error if
						// it is. Authorize will reject it anyway. We could
						// check if
						// greater than 30 days too
						if (order.getTotal() > payment.getAmount()) {
							result.put("resultCode", "-2");
							String resultMessage = "Found authorized payment but amount is less than current order total: "
									+ payment.getValue1()
									+ "; payment amount: "
									+ payment.getAmount()
									+ "; order total: " + order.getTotal();
							result.put("resultMessage", resultMessage);
							log.info(resultMessage);
							return result;
						}

						// Capture the payment now
						AuthorizeNetCIMResult transactionResult = null;
						Map parameters = new HashMap();
						parameters.put("authorizeTransID", payment.getValue1());
						transactionResult = acimp.processPriorAuthCapture(order, parameters);
						if (transactionResult.getError()) {
							Customer customer = null;
							if (getUser() != null && getUser().getCustomer() != null) {
								customer = getUser().getCustomer();
							}
							Map<String, String> error = acimp.logProfileErrors("Authorize.net transaction errors",
									transactionResult.getMessages(), customer, order, "processingPayment");
							String resultCode = null;
							String resultMessage = null;
							if (error != null) {
								resultCode = error.get("code");
								resultMessage = error.get("text");
							}
							if (resultCode == null) {
								resultCode = "XX";
							}
							result.put("resultCode", resultCode);
							result.put("resultMessage", resultMessage);
							return result;
						}

						// Must be a success
						if (payment.getPaymentProfileID() != null && !payment.getPaymentProfileID().equals("0")) {
							transactionResult.setPaymentProfileID(Long.parseLong(payment.getPaymentProfileID()));
						}
						parameters.put("order", order);
						return acimp.processSuccessResponse(parameters, transactionResult, true);
					}
				}
			}
			result.put("resultCode", "-3");
			result.put("resultMessage", "No authorized payment found for the order.");
		}
		return result;
	}
}
