package com.softslate.commerce.businessobjects.product;

import org.hibernate.envers.AuditTable;
import org.hibernate.envers.Audited;

import com.softslate.commerce.businessobjects.core.BusinessObject;

/**
 * Interface representing a product within a category in the system.
 * <p>
 * When the application needs to create an instance that implements
 * <code>ProductCategory</code>,
 * {@link com.softslate.commerce.businessobjects.core.BusinessObjectFactory}
 * finds the name of the Java class to instantiate from the
 * "productCategoryImplementer" setting in the
 * <code>appComponents.properties</code> file.
 * <p>
 * The default "productCategoryImplementer" is
 * {@link com.softslate.commerce.businessobjects.product.ProductCategoryBean}.
 * <p>
 * Why is this a separate entity in the domain model? It was added while
 * integrating Hibernate into the application. Our choices under Hibernate are
 * to: (1) Use a birectional &lt;many-to-many&gt; mapping, which would not
 * require a separate <code>ProductCategory</code> class. In this case the
 * productCategoryOrder field would have to be represented as the index column
 * in a &lt;list&gt; element. We voted this down because we felt making the
 * productCategoryOrder field refer to a strict indexed ordering (no gaps or
 * repetitions) is too restrictive when it comes to managing data imports and
 * updates in the Administrator. (2) We could represent the relationship using a
 * collection of components, ie a &lt;composite-element&gt; element in the
 * Hibernate mapping. But, this requires us to create the
 * <code>ProductCategory</code> class anyway, since the composite-element
 * would have to refer to a real class. (3) Our preference is to define the
 * <code>ProductCategory</code> element as a separate entity, with two
 * many-to-one relationships on either side. The major advantage is it has
 * maximum flexibility in terms of future growth, in case new columns need to be
 * added to the relationship table.
 * 
 * @author David Tobey
 * 
 */
@Audited
@AuditTable("AudProductCategory")
public interface ProductCategory extends BusinessObject {
	abstract int getProductCategoryID();

	abstract void setProductCategoryID(int productCategoryID);

	abstract int getCategoryID();

	abstract void setCategoryID(int categoryID);

	abstract int getProductID();

	abstract void setProductID(int productID);

	abstract int getProductCategoryOrder();

	abstract void setProductCategoryOrder(int productCategoryOrder);

	abstract Product getProduct();

	abstract void setProduct(Product product);

	abstract Category getCategory();

	abstract void setCategory(Category category);

}
