package com.softslate.commerce.customer.core;

import java.util.HashMap;
import java.util.Map;
import java.util.Properties;

import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.softslate.commerce.businessobjects.core.BusinessObjectFactory;
import com.softslate.commerce.businessobjects.core.Settings;
import com.softslate.commerce.businessobjects.core.User;
import com.softslate.commerce.businessobjects.customer.CustomerProcessor;
import com.softslate.commerce.businessobjects.order.OrderProcessor;

/**
 * Subclass of <code>BaseRequestProcessor</code> that negotiates each
 * application request for the customer interface. The
 * "/WEB-INF/conf/core/struts-config.xml" file identifies this class as the
 * customer application's &lt;controller&gt;.
 * 
 * @author David Tobey
 */
public class CustomerRequestProcessor extends CheckEERequestProcessor {
	static Log log = LogFactory.getLog(CustomerRequestProcessor.class);

	/**
	 * Runs initializations and other necessary routines before every request
	 * into the system.
	 * <p>
	 * Here are the steps this method takes for each request:
	 * <ol>
	 * <li>Initializes the request's <code>htmlHeadTitle</code> attribute to the
	 * "storeName" setting from the database. This attribute may be modified
	 * later by an action class or by
	 * {@link com.softslate.commerce.customer.core.LayoutAction}.</li>
	 * <li>Similarly initializes the request's <code>htmlHeadKeywords</code> and
	 * <code>htmlHeadDescription</code>attributes. These settings must be
	 * initialized in advance of forwarding the request to a Tiles definition so
	 * they are available when the "htmlHead" Tile is called.</li>
	 * <li>Sets the session scope's <code>currentLayout</code> attribute to the
	 * "layout" request parameter, if it is present, or to the "defaultLayout"
	 * setting from the database, if it is not present.</li>
	 * <li>Sets a convenience attribute named "loggedIn" to "true" in the
	 * request scope, if the current user is a logged in customer.</li>
	 * <li>Calls <code>sscOnCustomerPreRequestEnd</code>, which can be
	 * overridden by a subclass to provide customizations.</li>
	 * </ol>
	 * 
	 * @param request
	 *            The servlet request we are processing.
	 * @param response
	 *            The servlet response we are creating.
	 */
	protected boolean processPreprocess(HttpServletRequest request,
			HttpServletResponse response) {
		boolean proceed = super.processPreprocess(request, response);
		if (!proceed) {
			return false;
		}

		// Load the Settings from the application scope.
		Settings settings = (Settings) servlet.getServletContext()
				.getAttribute("settings");

		if (settings == null) {
			return true;
		}

		// Set the default HTML head title, keywords, and description.
		if (request.getAttribute("htmlHeadTitle") == null) {
			try {
				request.setAttribute("htmlHeadTitle", settings
						.getValue("storeName"));
				request.setAttribute("htmlHeadKeywords", settings
						.getValue("seoDefaultKeywords"));
				request.setAttribute("htmlHeadDescription", settings
						.getValue("seoDefaultDescription"));
			} catch (Exception e) {
				log.error(e.toString(), e);
			}
		}

		// If the user does not have an active cart but has a savedCartToken,
		// attempt to pick up the cart
		User user = (User) request.getSession().getAttribute("user");
		BusinessObjectFactory bof = (BusinessObjectFactory) request
				.getAttribute("businessObjectFactory");
		String automaticallySaveLastCart = (String) settings
				.getValue("automaticallySaveLastCart");
		String skipLoadSavedCart = (String) request
				.getAttribute("skipLoadSavedCart");
		if ((skipLoadSavedCart == null || skipLoadSavedCart.equals("false")) && user != null && user.getOrder() == null
				&& automaticallySaveLastCart != null && automaticallySaveLastCart.equals("1")) {

			synchronized (request.getSession()) {
				Cookie savedCartCookie = null;
				Cookie[] cookies = request.getCookies();
				if (cookies != null) {
					for (int i = 0; i < cookies.length; i++) {
						if (cookies[i] != null && cookies[i].getName() != null
								&& cookies[i].getName().equals("sscsctoken")) {
							savedCartCookie = cookies[i];
							break;
						}
					}
				}

				// Process the cart pickup
				if (savedCartCookie != null
						&& savedCartCookie.getValue() != null) {

					synchronized (request.getSession()) {
						Map parameters = new HashMap();
						parameters.put("autoSavedCartToken", savedCartCookie
								.getValue());
						try {
							OrderProcessor orderProcessor =  bof
									.createObject(OrderProcessor.class);
							Map results = orderProcessor
									.loadSavedCartFromToken(parameters);
							if (results != null) {
								request.getSession().setAttribute(
										"shippingAndTaxesProcessed", "false");
							}
						} catch (Exception e) {
							log.warn("Error attempting to auto load cart: ", e);
						}
					}
				}
			}
		}

		// If the user is not logged in but has an autoLoginToken,
		// attempt to log them in
		String allowCustomerAutoLogins = (String) settings
				.getValue("allowCustomerAutoLogins");
		String skipAutoLogin = (String) request.getAttribute("skipAutoLogin");
		if ((skipAutoLogin == null || skipAutoLogin.equals("false"))
				&& user.getCustomer() == null
				&& allowCustomerAutoLogins.equals("1")) {
			Cookie autoLoginCookie = null;
			Cookie[] cookies = request.getCookies();
			if (cookies != null) {
				for (int i = 0; i < cookies.length; i++) {
					if (cookies[i] != null && cookies[i].getName() != null
							&& cookies[i].getName().equals("sscaltoken")) {
						autoLoginCookie = cookies[i];
						break;
					}
				}
			}

			// Process the login
			if (autoLoginCookie != null && autoLoginCookie.getValue() != null) {

				synchronized (request.getSession()) {
					Map parameters = new HashMap();
					Map result = null;
					parameters
							.put("autoLoginToken", autoLoginCookie.getValue());
					try {
						CustomerProcessor customerProcessor =  bof
								.createObject(CustomerProcessor.class);
						result = customerProcessor.processAutoLogin(parameters);
					} catch (Exception e) {
						log.error("Error attempting auto login: ", e);
					}
					if (result != null && result.get("resultCode") != null
							&& result.get("resultCode").equals("0")) {
						request.getSession().setAttribute(
								"shippingAndTaxesProcessed", "false");
					}
				}
			}
		}

		// If the user is a logged in customer, set a request variable for
		// convenience.
		if (user != null && user.getCustomer() != null) {
			request.setAttribute("loggedIn", "true");
		}

		// Set other useful request parameters
		Properties appComponents = (Properties) servlet.getServletContext().getAttribute("appComponents");
		String activePaymentFormTemplates = appComponents.getProperty("activePaymentFormTemplates");
		request.setAttribute("activePaymentFormTemplates", activePaymentFormTemplates);
		String activePaymentProcessors = appComponents.getProperty("activePaymentProcessors");
		request.setAttribute("activePaymentProcessors", activePaymentProcessors);
		request.setAttribute("useCIM", activePaymentProcessors.contains("AuthorizeNetCIMProcessor"));
		request.setAttribute(
				"useCIMAPI",
				activePaymentProcessors.contains("AuthorizeNetCIMProcessor")
						&& ("API Only").equals(settings.getValue("authorizeNetCIMHostedMethod")));

		// Call a template method for hooks.
		boolean success = sscOnCustomerPreRequestEnd(request, response);

		return success;
	}

	/**
	 * Performs custom processing at the end of each call to
	 * <code>processPreProcess</code>. Intended to be overridden by subclasses
	 * for customizations. This method should return <code>false</code> to
	 * signal <code>processPreProcess</code> to return <code>false</code>
	 * itself.
	 * 
	 * @return <code>true</code> if the processing was successful,
	 *         <code>false</code> if processing should stop.
	 */
	public boolean sscOnCustomerPreRequestEnd(HttpServletRequest request,
			HttpServletResponse response) {
		BusinessObjectFactory bof = (BusinessObjectFactory) request
				.getAttribute("businessObjectFactory");
		if (bof != null) {
			ActionUtils actionUtils = (ActionUtils) bof
					.createUtilsObject("actionUtilsImplementer");
			return actionUtils.onCustomerPreRequestEnd(request, response);
		}
		return true;
	}
}
