package com.softslate.commerce.daos.customer;

import java.util.Collection;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.hibernate.HibernateException;
import org.hibernate.Query;
import org.hibernate.Session;

import com.softslate.commerce.businessobjects.core.BusinessObjectFactory;
import com.softslate.commerce.businessobjects.core.User;
import com.softslate.commerce.businessobjects.customer.Customer;
import com.softslate.commerce.businessobjects.customer.CustomerAddress;
import com.softslate.commerce.daos.core.BaseAdminGatewayDAOHibernate;
import com.softslate.commerce.daos.core.DataAccessException;
import com.softslate.commerce.daos.order.OrderDAO;

/**
 * Generic DAO that runs miscellaneous queries related to customers.
 * 
 * @author David Tobey
 */
public class CustomerGatewayDAOHibernate extends BaseAdminGatewayDAOHibernate
		implements CustomerGatewayDAO {

	static Log log = LogFactory.getLog(CustomerGatewayDAOHibernate.class);

	public Map processRegister(User user) throws Exception {
		if (log.isInfoEnabled())
			log.info("Processing customer registration.");
		Map result = new HashMap();
		CustomerDAO customerDAO = (CustomerDAO) getDaoFactory().createDAO(
				"customerDAOImplementer");
		customerDAO.setCustomer(user.getCustomer());
		if (customerDAO.getUserNameCount() > 0) {
			result.put("resultCode", "errors.userNameNotUnique");
			return result;
		}
		if (log.isInfoEnabled())
			log.info("Inserting new customer: "
					+ user.getCustomer().getUserName());
		customerDAO.insertCustomer();
		getDaoFactory().commitTransaction();
		if (user.getCustomer() != null) {
			getDaoFactory().getSession().evict(user.getCustomer());
		}
		result.put("resultCode", "0");
		return result;
	}

	public Map processLogin(User user) throws Exception {
		if (log.isInfoEnabled())
			log.info("Processing customer login.");
		Map result = new HashMap();
		CustomerDAO customerDAO = (CustomerDAO) getDaoFactory().createDAO(
				"customerDAOImplementer");
		customerDAO.setCustomer(user.getCustomer());
		// Save the autoLoginToken, if there is one
		String autoLoginToken = user.getCustomer().getAutoLoginToken();
		// Load the customer
		customerDAO.loadFromUserNameAndPassword();
		if (customerDAO.getCustomer() == null) {
			user.setCustomer(null);
			result.put("resultCode", "errors.invalidLogin");
			return result;
		}
		user.setCustomer(customerDAO.getCustomer());
		// Load the customer addresses and wish lists
		Collection addresses = customerDAO.loadAddressCollection();
		// Load the customer wish lists
		Collection wishLists = customerDAO.loadCustomerWishLists();
		customerDAO.getCustomer().setLastLogin(formatDateTime(new Date()));
		customerDAO.getCustomer().setAutoLoginToken(autoLoginToken);
		// Add the customer's ID and user name to the order, if one exists.
		if (user.getOrder() != null) {
			OrderDAO orderDAO = (OrderDAO) getDaoFactory().createDAO(
					"orderDAOImplementer");
			orderDAO.setOrder(user.getOrder());
			user.getOrder().setCustomer(user.getCustomer());
			user.getOrder().setUserName(user.getCustomer().getUserName());
			orderDAO.updateOrder();
		}
		getDaoFactory().commitTransaction();
		result.put("wishLists", wishLists);
		result.put("addresses", addresses);
		result.put("resultCode", "0");
		return result;
	}

	public Map processAutoLogin(User user) throws Exception {
		if (log.isInfoEnabled())
			log.info("Processing auto customer login.");
		Map result = new HashMap();
		CustomerDAO customerDAO = (CustomerDAO) getDaoFactory().createDAO(
				"customerDAOImplementer");
		customerDAO.setCustomer(user.getCustomer());
		// Load the customer
		customerDAO.loadFromAutoLoginToken();
		if (customerDAO.getCustomer() == null) {
			user.setCustomer(null);
			result.put("resultCode", "errors.invalidLogin");
			return result;
		}
		user.setCustomer(customerDAO.getCustomer());
		// Load the customer addresses and wish lists
		Collection addresses = customerDAO.loadAddressCollection();
		// Load the customer wish lists
		Collection wishLists = customerDAO.loadCustomerWishLists();
		customerDAO.getCustomer().setLastLogin(formatDateTime(new Date()));
		// Add the customer's ID and user name to the order, if one exists.
		if (user.getOrder() != null) {
			OrderDAO orderDAO = (OrderDAO) getDaoFactory().createDAO(
					"orderDAOImplementer");
			orderDAO.setOrder(user.getOrder());
			user.getOrder().setCustomer(user.getCustomer());
			user.getOrder().setUserName(user.getCustomer().getUserName());
			orderDAO.updateOrder();
		}
		getDaoFactory().commitTransaction();
		result.put("wishLists", wishLists);
		result.put("addresses", addresses);
		result.put("resultCode", "0");
		return result;
	}

	public Map refreshCustomerFromDatabase(User user) throws Exception {
		if (log.isInfoEnabled())
			log.info("Refreshing the customer object.");
		Map result = new HashMap();
		CustomerDAO customerDAO =  getDaoFactory().createDAO(CustomerDAO.class);
		customerDAO.setCustomer(user.getCustomer());
		// Save the autoLoginToken, if there is one
		String autoLoginToken = user.getCustomer().getAutoLoginToken();
		// Load the customer by the ID
		customerDAO.loadCustomerFromID();
		if (customerDAO.getCustomer() == null) {
			user.setCustomer(null);
			result.put("resultCode", "errors.invalidLogin");
			return result;
		}
		user.setCustomer(customerDAO.getCustomer());
		// Load the customer addresses and wish lists
		Collection addresses = customerDAO.loadAddressCollection();
		// Load the customer wish lists
		Collection wishLists = customerDAO.loadCustomerWishLists();
		customerDAO.getCustomer().setAutoLoginToken(autoLoginToken);
		result.put("wishLists", wishLists);
		result.put("addresses", addresses);
		result.put("resultCode", "0");
		return result;
	}

	public Map updateAddresses(Customer customer) throws Exception {
		// Hibernate takes care of address records.
		return updateCustomer(customer, false);
	}

	public void updateCustomerAndAddress(Customer customer,
			CustomerAddress customerAddress) throws Exception {
		// Hibernate takes care of address records.
		updateCustomer(customer, false);
		return;
	}

	public void updateCustomerInsertAddress(Customer customer,
			CustomerAddress customerAddress) throws Exception {
		// Hibernate takes care of address records.
		updateCustomer(customer, false);
		return;
	}

	public Map updateCustomer(Customer customer, boolean forcePassword)
			throws Exception {
		Map result = new HashMap();
		CustomerDAO customerDAO = (CustomerDAO) getDaoFactory().createDAO(
				"customerDAOImplementer");
		customerDAO.setCustomer(customer);
		if (customerDAO.getUserNameCount() > 0) {
			result.put("resultCode", "errors.userNameNotUnique");
			return result;
		}
		if (log.isInfoEnabled())
			log.info("updateCustomer");
		customerDAO.updateCustomer(false, forcePassword);
		getDaoFactory().commitTransaction();
		if (customer != null) {
			getDaoFactory().getSession().evict(customer);
		}
		result.put("resultCode", "0");
		return result;
	}

	public Map loadCustomersAndCount(Map parameters) throws Exception {
		return loadItemsAndCount(parameters);
	}

	public Map editDeleteCustomers(Map parameters, BusinessObjectFactory bof) throws Exception {
		return editDeleteItems(parameters, bof);
	}

	public int loadMaxCustomerID() throws Exception {
		Session session = getDaoFactory().startSession();
		try {
			Query q = getNamedQuery(session, "customer.maxCustomerID");
			Integer maxNum = (Integer) q.uniqueResult();
			if (maxNum == null) {
				return 0;
			}
			return maxNum.intValue();
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
	}

	public Collection loadAllCustomersInRange(int firstID, int lastID)
			throws Exception {
		if (log.isInfoEnabled())
			log.info("Loading customers within an id range: " + firstID + "-"
					+ lastID);
		Collection customers = null;
		Session session = getDaoFactory().startSession();
		try {
			Query q = getNamedQuery(session, "customer.selectAllInRange");
			q.setInteger("firstID", firstID);
			q.setInteger("lastID", lastID);
			customers = new LinkedHashSet(q.list());
			if (null != customers) {
				Iterator i = customers.iterator();
				while (i.hasNext()) {
					Customer c = (Customer) i.next();
					if (c.getPassword() != null) {
						String decrypted = getDAOUtils().getDecrypted(
								c.getPassword(),
								(String) getSettings().getValue(
										"customerEncryptionType"),
								getAppSettings().getProperty("twoWayKeyFile"));
						c.setDecryptedPassword(decrypted);
					}
				}
			}
			return customers;
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
	}
	
	public Collection<Customer> loadCustomersWithNoCIMProfileByDate(
			String cutoffDate) throws Exception {
		Session session = getDaoFactory().startSession();
		try {
			Query q = getNamedQuery(session,
					"customer.selectCustomersWithNoCIMProfileByDate");
			q.setString("cutoffDate", cutoffDate);
			return new LinkedHashSet<Customer>(q.list());
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
	}
	
	public List<Object[]> loadCustomersWithNoCIMProfileByDate(
			String createdStart, String createdEnd) throws Exception {
		Session session = getDaoFactory().startSession();
		try {
			Query q = getNamedQuery(session,
					"customer.selectCustomersWithNoCIMProfileByDate");
			q.setString("createdStart", createdStart);
			q.setString("createdEnd", createdEnd);
			List<Object[]> list = q.list();
			return list;
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
	}

}
