package com.softslate.commerce.daos.product;

import java.util.Collection;
import java.util.Collections;
import java.util.Iterator;
import java.util.LinkedHashSet;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.hibernate.Criteria;
import org.hibernate.Hibernate;
import org.hibernate.HibernateException;
import org.hibernate.Query;
import org.hibernate.Session;
import org.hibernate.criterion.Restrictions;
import org.hibernate.type.IntegerType;
import org.hibernate.type.StringType;

import com.softslate.commerce.businessobjects.core.BusinessObjectFactory;
import com.softslate.commerce.businessobjects.product.Category;
import com.softslate.commerce.businessobjects.product.CategorySetting;
import com.softslate.commerce.businessobjects.product.Product;
import com.softslate.commerce.businessobjects.product.ProductList;
import com.softslate.commerce.businessobjects.product.ProductListBean;
import com.softslate.commerce.businessobjects.product.ProductProcessor;
import com.softslate.commerce.businessobjects.product.ProductSetting;
import com.softslate.commerce.daos.core.BaseAdminGatewayDAOHibernate;
import com.softslate.commerce.daos.core.DataAccessException;

/**
 * Generic DAO that runs miscellaneous queries related to products and
 * categories.
 * 
 * @author David Tobey
 */
public class ProductGatewayDAOHibernate extends BaseAdminGatewayDAOHibernate
		implements ProductGatewayDAO {

	static Log log = LogFactory.getLog(ProductGatewayDAOHibernate.class);

	public Collection loadCategoryTree() throws DataAccessException {
		if (log.isInfoEnabled())
			log.info("Loading category tree.");
		Collection categoryTree = null;
		Session session = getDaoFactory().startSession();
		session.clear();
		try {
			Query q = getNamedQuery(session, "category.selectTopLevel");
			q.setInteger("categoryTreeID", 1);
			categoryTree = new LinkedHashSet(q.list());
			// Because lazy fetching for the subcategory collection is enabled
			// in the mapping file, we must recursively initialize the
			// subcategory tree manually here
			Iterator i = categoryTree.iterator();
			while (i.hasNext()) {
				Category category = (Category) i.next();
				category.setCategoryLevel(1);
				if (log.isDebugEnabled())
					log.debug("Category found: " + category.getCode());
				if (category.getSubcategoryCollection() != null) {
					Hibernate.initialize(category.getSubcategoryCollection());
					initializeSubcategories(category);
				}
			}
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
		return categoryTree;
	}

	private void initializeSubcategories(Category category) {

		// Limit the number of levels the parent categories recurse to avoid
		// an infinite loop
		int categoryLevel = category.getCategoryLevel() + 1;
		int maxRecordsProcessed = 100;
		try {
			maxRecordsProcessed = Integer.parseInt((String) getSettings()
					.getValue("maxRecordsProcessed"));
		} catch (Exception e) {
			log.warn("maxRecordsProcessed error, defaulting to 100: ", e);
		}
		if (categoryLevel >= maxRecordsProcessed) {
			log.warn("Max number of category levels reached ("
					+ maxRecordsProcessed + ") with: " + category.getCode()
					+ ". Check for circular references.");
			return;
		}

		// Loop through the subcategories
		Collection subcategories = category.getSubcategoryCollection();
		Iterator i = subcategories.iterator();
		while (i.hasNext()) {
			Category subcategory = (Category) i.next();
			subcategory.setCategoryLevel(categoryLevel);
			if (log.isDebugEnabled())
				log.debug("Subcategory found: " + subcategory.getCode());
			if (subcategory.getSubcategoryCollection() != null) {
				Hibernate.initialize(subcategory.getSubcategoryCollection());
				initializeSubcategories(subcategory);
			}
		}
	}

	public ProductList loadProductList(ProductList productList)
			throws DataAccessException {
		if (log.isInfoEnabled())
			log.info("Loading product list.");
		int productListType = productList.getProductListType();
		Session session = getDaoFactory().startSession();
		Query q = null;
		try {
			switch (productListType) {
			case ProductListBean.CATEGORY:
				q = getNamedQuery(session, "category.selectProductList");
				q.setCacheable(true);
				q.setInteger("categoryID", productList.getCategoryID());
				q.setMaxResults(productList.getItemsPerPage());
				if (productList.getFirstRow() > 1) {
					q.setFirstResult(productList.getFirstRow() - 1);
				}
				Collection products = new LinkedHashSet(q.list());
				productList.setProductListCollection(products);
				break;
			case ProductListBean.MANUFACTURER:
				q = getNamedQuery(session, "manufacturer.selectProductList");
				q.setCacheable(true);
				q.setInteger("manufacturerID", productList.getManufacturerID());
				q.setMaxResults(productList.getItemsPerPage());
				if (productList.getFirstRow() > 1) {
					q.setFirstResult(productList.getFirstRow() - 1);
				}
				Collection manfProducts = new LinkedHashSet(q.list());
				productList.setProductListCollection(manfProducts);
				break;
			case ProductListBean.SEARCH:
				productList.setSearchString(getDAOUtils().prepareSearchString(
						productList.getSearchString()));
				if (log.isDebugEnabled())
					log.debug("productList.getSearchString(): "
							+ productList.getSearchString());
				q = getNamedQuery(session, "search.products");
				q.setString("searchString", productList.getSearchString());
				if (productList.getFirstRow() > 1) {
					q.setFirstResult(productList.getFirstRow() - 1);
				}
				q.setMaxResults(productList.getItemsPerPage());
				productList
						.setProductListCollection(new LinkedHashSet(q.list()));
				break;
			default: // ALL_ACTIVE is the default.
				q = getNamedQuery(session, "productList.selectAllActive");
				q.setCacheable(true);
				if (productList.getFirstRow() > 1) {
					q.setFirstResult(productList.getFirstRow() - 1);
				}
				q.setMaxResults(productList.getItemsPerPage());
				productList
						.setProductListCollection(new LinkedHashSet(q.list()));
			}
			if (log.isDebugEnabled())
				log.debug("Product list length: "
						+ Integer.toString(productList
								.getProductListCollection().size()));
			// Get a count of the total number of products.
			Query q2 = null;
			switch (productListType) {
			case ProductListBean.CATEGORY:
				q2 = getNamedQuery(session, "category.countProductList");
				q2.setCacheable(true);
				q2.setInteger("categoryID", productList.getCategoryID());
				break;
			case ProductListBean.MANUFACTURER:
				q2 = getNamedQuery(session, "manufacturer.countProductList");
				q2.setCacheable(true);
				q2
						.setInteger("manufacturerID", productList
								.getManufacturerID());
				break;
			case ProductListBean.SEARCH:
				q2 = getNamedQuery(session, "search.countProducts");
				q2.setString("searchString", productList.getSearchString());
				break;
			default: // ALL_ACTIVE is the default.
				q2 = getNamedQuery(session, "productList.countAllActive");
				q2.setCacheable(true);
			}
			Long count = (Long) q2.uniqueResult();
			productList.setItemCount(count.intValue());
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
		return productList;
	}

	public Map loadProductsAndCount(Map parameters) throws Exception {
		return loadItemsAndCount(parameters);
	}

	public Collection loadItems(Map parameters) throws DataAccessException {
		String parentID = (String) parameters.get("parentID");
		String assignments = (String) parameters.get("assignments");
		Collection assignedItemsList = (Collection) parameters
				.get("assignedItemsList");
		String searchString = (String) parameters.get("searchString");
		String sortByString = (String) parameters.get("sortByString");
		String sortAscOrDesc = (String) parameters.get("sortAscOrDesc");
		String firstRow = (String) parameters.get("firstRow");
		String itemsPerPage = (String) parameters.get("itemsPerPage");
		String manyToManyProp = (String) parameters.get("manyToManyProp");
		String[] searchableFields = (String[]) parameters
				.get("searchableFields");
		String[] searchableIntFields = (String[]) parameters
				.get("searchableIntFields");

		if (assignments != null && assignments.equals("assigned")
				&& manyToManyProp.equals("productCategories")) {
			// Debugging
			if (log.isDebugEnabled())
				log.debug("assignments: " + assignments);
			if (log.isDebugEnabled())
				log.debug("assignedItemsList: " + assignedItemsList);
			if (log.isDebugEnabled())
				log.debug("parentID: " + parentID);
			if (log.isDebugEnabled())
				log.debug("Search string: " + searchString);
			if (log.isDebugEnabled())
				log.debug("sortByString: "
						+ (String) parameters.get("sortByString"));
			if (log.isDebugEnabled())
				log.debug("sortAscOrDesc: "
						+ (String) parameters.get("sortAscOrDesc"));
			if (log.isDebugEnabled())
				log.debug("firstRow (+1): " + firstRow);
			if (log.isDebugEnabled())
				log.debug("itemsPerPage: " + itemsPerPage);

			Collection results = null;
			Session session = getDaoFactory().startSession();
			try {

				// Create the query in HQL in order to pull products with the
				// category as their primary category
				StringBuffer query = new StringBuffer("");
				query.append("SELECT p FROM Product p LEFT JOIN "
						+ "p.productCategories pc WITH "
						+ "pc.categoryID = :categoryID "
						+ "WHERE (p.primaryCategoryID = "
						+ ":categoryID OR pc.categoryID = " + ":categoryID)");

				// Create the filter section of the where clause
				query = processCommonHQLSearchClause(query, parameters);

				// Set the ORDER BY parameters
				if (sortByString != null && sortByString.length() > 0) {
					query.append(" ORDER BY " + sortByString);
					if (sortAscOrDesc != null && sortAscOrDesc.equals("DESC")) {
						query.append(" DESC");
					}
				}

				// Create the Query object and define the parameters
				Query hibQuery = session.createQuery(query.toString());
				hibQuery.setInteger("categoryID", new Integer(parentID)
						.intValue());
				if (searchString != null
						&& searchString.length() > 0
						&& ((searchableFields != null && searchableFields.length > 0) || (searchableIntFields != null && searchableIntFields.length > 0))) {
					searchString = getDAOUtils().prepareSearchString(
							searchString);
					hibQuery.setString("searchString", searchString);
				}

				// Set the paging parameters
				if (firstRow != null && itemsPerPage != null) {
					hibQuery.setFirstResult(Integer.parseInt(firstRow) - 1);
					hibQuery.setMaxResults(Integer.parseInt(itemsPerPage));
				}

				// Run the query
				results = new LinkedHashSet(hibQuery.list());
			} catch (HibernateException he) {
				if (log.isErrorEnabled())
					log.error("Hibernate error: ", he);
				throw new DataAccessException(he);
			}
			return results;

		} else {
			return super.loadItems(parameters);
		}
	}

	public int loadItemsCount(Map parameters) throws DataAccessException {
		String parentID = (String) parameters.get("parentID");
		String assignments = (String) parameters.get("assignments");
		Collection assignedItemsList = (Collection) parameters
				.get("assignedItemsList");
		String searchString = (String) parameters.get("searchString");
		// Debugging
		if (log.isDebugEnabled())
			log.debug("assignments: " + assignments);
		if (log.isDebugEnabled())
			log.debug("assignedItemsList: " + assignedItemsList);
		if (log.isDebugEnabled())
			log.debug("parentID: " + parentID);
		if (log.isDebugEnabled())
			log.debug("Search string: " + searchString);
		String[] searchableFields = (String[]) parameters
				.get("searchableFields");
		String[] searchableIntFields = (String[]) parameters
				.get("searchableIntFields");
		String manyToManyProp = (String) parameters.get("manyToManyProp");

		if (assignments != null && assignments.equals("assigned")
				&& manyToManyProp.equals("productCategories")) {

			Long result = null;
			Session session = getDaoFactory().startSession();
			try {

				// Create the query in HQL in order to pull products with the
				// category as their primary category
				StringBuffer query = new StringBuffer("");
				query
						.append("SELECT COUNT(DISTINCT p) FROM Product p LEFT JOIN "
								+ "p.productCategories pc WHERE (p.primaryCategoryID = "
								+ ":categoryID OR pc.categoryID = :categoryID )");

				// Create the filter section of the where clause
				query = processCommonHQLSearchClause(query, parameters);

				// Create the Query object and define the parameters
				Query hibQuery = session.createQuery(query.toString());
				hibQuery.setInteger("categoryID", new Integer(parentID)
						.intValue());
				if (searchString != null
						&& searchString.length() > 0
						&& ((searchableFields != null && searchableFields.length > 0) || (searchableIntFields != null && searchableIntFields.length > 0))) {
					searchString = getDAOUtils().prepareSearchString(
							searchString);
					hibQuery.setString("searchString", searchString);
				}

				// Run the query
				result = (Long) hibQuery.uniqueResult();
			} catch (HibernateException he) {
				if (log.isErrorEnabled())
					log.error("Hibernate error: ", he);
				throw new DataAccessException(he);
			}
			return result.intValue();

		} else {
			return super.loadItemsCount(parameters);
		}
	}

	public StringBuffer processCommonHQLSearchClause(StringBuffer query,
			Map parameters) {

		String searchString = (String) parameters.get("searchString");
		String[] searchableFields = (String[]) parameters
				.get("searchableFields");
		String[] searchableIntFields = (String[]) parameters
				.get("searchableIntFields");

		// Set the filter HQL
		if (searchString != null && searchString.length() > 0) {
			query.append(" AND (");
			// Test for a match on integer fields
			if (searchableIntFields != null) {
				try {
					Integer searchInteger = new Integer(Integer
							.parseInt(searchString));
					for (int i = 0; i < searchableIntFields.length; i++) {
						if (i > 0) {
							query.append(" OR ");
						}
						query.append(searchableIntFields[i] + " = "
								+ searchInteger);
					}
				} catch (NumberFormatException nfe) {
					// Swallow
				}
			}
			// Test for a match on string fields
			if (searchableFields != null) {
				for (int i = 0; i < searchableFields.length; i++) {
					if (i > 0) {
						query.append(" OR ");
					}
					query.append("LOWER(" + searchableFields[i]
							+ ") LIKE :searchString ");
				}
			}
			query.append(" )");
		}
		return query;
	}

	public Map editDeleteProducts(Map parameters, BusinessObjectFactory bof) throws Exception {
		Map result = editDeleteItems(parameters, bof);
		evictCache();
		return result;
	}

	public Criteria processCommonCriteria(Criteria crit, Map parameters) {

		// If we're displaying a product's related products, exclude the parent
		// product itself from the list.
		crit = super.processCommonCriteria(crit, parameters);
		String parentID = (String) parameters.get("parentID");
		String parentIDField = (String) parameters.get("parentIDField");
		if (parentIDField != null && parentIDField.equals("parentProductID")) {
			crit.add(Restrictions.ne("productID", new Integer(Integer
					.parseInt(parentID))));
		}
		return crit;
	}

	public Map postChangingItemEvent(Object item, Map parameters, BusinessObjectFactory bof) throws Exception {
		ProductProcessor processor = bof.createObject(ProductProcessor.class);
		Map results = processor.postChangingProductEvent((Product) item, parameters);
		return results;
	}

	public Collection loadAllProducts() throws DataAccessException {
		Collection products = null;
		Session session = getDaoFactory().startSession();
		try {
			Query q = getNamedQuery(session, "product.selectAll");
			products = new LinkedHashSet(q.list());
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
		return products;
	}

	public int loadMaxProductID() throws Exception {
		Session session = getDaoFactory().startSession();
		try {
			Query q = getNamedQuery(session, "product.maxProductID");
			Integer maxNum = (Integer) q.uniqueResult();
			if (maxNum == null) {
				return 0;
			}
			return maxNum.intValue();
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
	}

	public Collection loadProductsForIndexing(int firstProductID,
			int lastProductID) throws DataAccessException {
		Collection products = null;
		Session session = getDaoFactory().startSession();
		try {
			Query q = getNamedQuery(session, "product.selectForIndexing");
			q.setInteger("firstProductID", firstProductID);
			q.setInteger("lastProductID", lastProductID);
			products = new LinkedHashSet(q.list());
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
		return products;
	}

	public Collection loadAllProductsSortedByName() throws Exception {
		Collection results = null;
		Session session = getDaoFactory().startSession();
		try {
			Query q = getNamedQuery(session, "product.selectAllSortedByName");
			results = new LinkedHashSet(q.list());
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
		}
		return results;
	}

	public Collection loadAllProductsSortedByCode() throws Exception {
		Collection results = null;
		Session session = getDaoFactory().startSession();
		try {
			Query q = getNamedQuery(session, "product.selectAllSortedByCode");
			results = new LinkedHashSet(q.list());
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
		}
		return results;
	}

	public Long loadProductCount() throws DataAccessException {
		Session session = getDaoFactory().startSession();
		try {
			Query q = getNamedQuery(session, "product.selectProductCount");
			return (Long) q.uniqueResult();
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
	}

	public Collection loadAllProductNamesCodesIDs() throws DataAccessException {
		Collection products = null;
		Session session = getDaoFactory().startSession();
		try {
			Query q = getNamedQuery(session, "product.selectNamesCodesIDs");
			products = new LinkedHashSet(q.list());
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
		return products;
	}

	public Collection loadActiveProductNamesCodesIDs()
			throws DataAccessException {
		Collection products = null;
		Session session = getDaoFactory().startSession();
		try {
			Query q = getNamedQuery(session,
					"product.selectActiveNamesCodesIDs");
			products = new LinkedHashSet(q.list());
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
		return products;
	}

	public Collection processSettings(Map parameters) throws Exception {
		return processSettings(false, parameters);
	}

	public Collection processSettings(boolean commit, Map parameters)
			throws Exception {
		if (log.isInfoEnabled())
			log.info("Processing product settings.");
		ProductSettingDAO productSettingDAO =  getDaoFactory()
				.createDAO(ProductSettingDAO.class);
		ProductDAO productDAO = (ProductDAO) parameters.get("productDAO");
		// Update all the existing settings
		if (log.isDebugEnabled())
			log.debug("Processing product setting updates.");
		Collection settings = (Collection) parameters.get("settings");
		String[] codes = (String[]) parameters.get("code");
		String[] values = (String[]) parameters.get("value");
		String[] valueTypes = (String[]) parameters.get("valueType");
		String[] names = (String[]) parameters.get("name");
		String[] descriptions = (String[]) parameters.get("description");
		String[] integerValue = (String[]) parameters.get("integerValue");
		String[] smallValue = (String[]) parameters.get("smallValue");
		String[] mediumValue = (String[]) parameters.get("mediumValue");
		String[] textValue = (String[]) parameters.get("textValue");
		for (int i = 0; i < codes.length; i++) {
			Iterator j = settings.iterator();
			while (j.hasNext()) {
				ProductSetting setting = (ProductSetting) j.next();
				if (setting.getCode().equals(codes[i])) {
					if (log.isDebugEnabled())
						log.debug("Existing product setting: " + codes[i]);
					if (names != null && names[i] != null) {
						setting.setName(names[i]);
					}
					if (descriptions != null && descriptions[i] != null) {
						setting.setDescription(descriptions[i]);
					}
					if (integerValue != null && integerValue[i] != null && integerValue[i].length() > 0) {
						try {
							setting.setIntegerValue(Integer.parseInt(integerValue[i]));
						} catch (NumberFormatException nfe) {
							// swallow
						}
					}
					if (smallValue != null && smallValue[i] != null) {
						setting.setSmallValue(smallValue[i]);
					}
					if (mediumValue != null && mediumValue[i] != null) {
						setting.setMediumValue(mediumValue[i]);
					}
					if (textValue != null && textValue[i] != null) {
						setting.setTextValue(textValue[i]);
					}
					if (valueTypes[i].equals("integerValue")
							|| valueTypes[i].equals("booleanValue")) {
						setting.setIntegerValue(Integer.parseInt(values[i]));
					} else if (valueTypes[i].equals("smallValue")) {
						setting.setSmallValue(values[i]);
					} else if (valueTypes[i].equals("mediumValue")) {
						setting.setMediumValue(values[i]);
					} else if (valueTypes[i].equals("textValue")) {
						setting.setTextValue(values[i]);
					}
				}
			}
		}
		// Insert the new settings
		if (log.isDebugEnabled())
			log.debug("Processing product setting inserts.");
		Collection productSettingsToInsert = (Collection) parameters
				.get("productSettingsToInsert");
		Iterator i = productSettingsToInsert.iterator();
		while (i.hasNext()) {
			productSettingDAO.setProductSetting((ProductSetting) i.next());
			productSettingDAO.insertProductSetting();
		}
		// Reload the settings and return them
		Collection newSettings = productDAO.loadSettings();
		if (commit) {
			getDaoFactory().commitTransaction();
		}
		return newSettings;
	}

	public Collection processCategorySettings(Map parameters) throws Exception {
		return processCategorySettings(false, parameters);
	}

	public Collection processCategorySettings(boolean commit, Map parameters)
			throws Exception {
		if (log.isInfoEnabled())
			log.info("Processing category settings.");
		CategorySettingDAO categorySettingDAO =  getDaoFactory()
				.createDAO(CategorySettingDAO.class);
		CategoryDAO categoryDAO = (CategoryDAO) parameters.get("categoryDAO");
		// Update all the existing settings
		if (log.isDebugEnabled())
			log.debug("Processing category setting updates.");
		Collection settings = (Collection) parameters.get("settings");
		String[] codes = (String[]) parameters.get("code");
		String[] names = (String[]) parameters.get("name");
		String[] descriptions = (String[]) parameters.get("description");
		String[] values = (String[]) parameters.get("value");
		String[] valueTypes = (String[]) parameters.get("valueType");
		for (int i = 0; i < codes.length; i++) {
			Iterator j = settings.iterator();
			while (j.hasNext()) {
				CategorySetting setting = (CategorySetting) j.next();
				if (setting.getCode().equals(codes[i])) {
					if (log.isDebugEnabled())
						log.debug("Existing category setting: " + codes[i]);
					if (valueTypes[i].equals("integerValue")
							|| valueTypes[i].equals("booleanValue")) {
						setting.setIntegerValue(Integer.parseInt(values[i]));
					} else if (valueTypes[i].equals("smallValue")) {
						setting.setSmallValue(values[i]);
					} else if (valueTypes[i].equals("mediumValue")) {
						setting.setMediumValue(values[i]);
					} else if (valueTypes[i].equals("textValue")) {
						setting.setTextValue(values[i]);
					}
					if (names != null && names.length > i) {
						setting.setName(names[i]);
					}
					if (descriptions != null && descriptions.length > i) {
						setting.setDescription(descriptions[i]);
					}
				}
			}
		}
		// Insert the new settings
		if (log.isDebugEnabled())
			log.debug("Processing category setting inserts.");
		Collection categorySettingsToInsert = (Collection) parameters
				.get("categorySettingsToInsert");
		Iterator i = categorySettingsToInsert.iterator();
		while (i.hasNext()) {
			categorySettingDAO.setCategorySetting((CategorySetting) i.next());
			categorySettingDAO.insertCategorySetting();
		}
		// Reload the settings and return them
		Collection newSettings = categoryDAO.loadSettings();
		if (commit) {
			getDaoFactory().commitTransaction();
		}
		return newSettings;
	}

	public Collection loadProductSettings(Collection productIDs)
			throws Exception {
		Collection items = null;
		Session session = getDaoFactory().startSession();
		try {
			Query q = getNamedQuery(session, "productSetting.selectMultiple");
			q.setParameterList("productIDs", productIDs, IntegerType.INSTANCE);
			q.setCacheable(true);
			items = new LinkedHashSet(q.list());
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
		return items;
	}

	public Collection loadSkus(Collection productIDs, Collection attributeIDs)
			throws Exception {
		if (log.isInfoEnabled())
			log.info("Loading skus.");

		// Work around Hibnerate parsing error if collection is empty
		if (attributeIDs.isEmpty()) {
			attributeIDs = Collections.singleton(Integer.valueOf(-1));
		}
		Collection items = null;
		Session session = getDaoFactory().startSession();
		try {
			Query q = getNamedQuery(session, "sku.selectFromProductIDList");
			q.setParameterList("productIDs", productIDs, IntegerType.INSTANCE);
			q.setParameterList("attributeIDs", attributeIDs, IntegerType.INSTANCE);
			q.setCacheable(true);
			items = new LinkedHashSet(q.list());
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
		return items;
	}

	public void updateProductsActive(Collection productIDs, boolean isActive)
			throws Exception {
		updateProductsActive(false, productIDs, isActive);
	}

	public void updateProductsActive(boolean commit, Collection productIDs,
			boolean isActive) throws Exception {
		if (log.isInfoEnabled())
			log.info("Updating products to be in/active.");
		Collection productIDIntegers = getDAOUtils().getIntegersFromStrings(
				productIDs);
		Session session = getDaoFactory().startSession();
		try {
			Query q = getNamedQuery(session, "product.updateProductsActive");
			if (isActive) {
				q.setBoolean("isActive", true);
			} else {
				q.setBoolean("isActive", false);
			}
			q.setParameterList("productIDs", productIDIntegers,
 IntegerType.INSTANCE);
			q.executeUpdate();
			evictCache();
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
	}

	public Collection loadAttributeSKUs() throws DataAccessException {
		Collection skus = null;
		Session session = getDaoFactory().startSession();
		try {
			Query q = getNamedQuery(session, "sku.selectAttributeOnly");
			q.setCacheable(true);
			skus = new LinkedHashSet(q.list());
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
		return skus;
	}

	public void updateProductCategoryOrders() throws DataAccessException {
		Collection results = null;
		Session session = getDaoFactory().startSession();
		try {
			Query q = getNamedQuery(session,
					"productCategory.selectOutOfSyncOrders");
			results = new LinkedHashSet(q.list());
			if (results != null) {
				Query q2 = getNamedQuery(session,
						"productCategory.updateOutOfSyncOrder");
				Iterator i = results.iterator();
				while (i.hasNext()) {
					Object[] result = (Object[]) i.next();
					Integer productOrder = (Integer) result[0];
					Integer productCategoryID = (Integer) result[1];
					q2.setInteger("productOrder", productOrder.intValue());
					q2.setInteger("productCategoryID", productCategoryID
							.intValue());
					q2.executeUpdate();
					evictCache();
				}
			}
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
		return;
	}

	public void updateProductAttributeOrders() throws DataAccessException {
		Collection results = null;
		Session session = getDaoFactory().startSession();
		try {
			Query q = getNamedQuery(session,
					"productAttribute.selectOutOfSyncOrders");
			results = new LinkedHashSet(q.list());
			if (results != null) {
				Query q2 = getNamedQuery(session,
						"productAttribute.updateOutOfSyncOrder");
				Iterator i = results.iterator();
				while (i.hasNext()) {
					Object[] result = (Object[]) i.next();
					Integer attributeOrder = (Integer) result[0];
					Integer productAttributeID = (Integer) result[1];
					q2.setInteger("attributeOrder", attributeOrder.intValue());
					q2.setInteger("productAttributeID", productAttributeID
							.intValue());
					q2.executeUpdate();
				}
			}
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
		return;
	}

	public void updateRelatedProductOrders() throws DataAccessException {
		Collection results = null;
		Session session = getDaoFactory().startSession();
		try {
			Query q = getNamedQuery(session,
					"relatedProduct.selectOutOfSyncOrders");
			results = new LinkedHashSet(q.list());
			if (results != null) {
				Query q2 = getNamedQuery(session,
						"relatedProduct.updateOutOfSyncOrder");
				Iterator i = results.iterator();
				while (i.hasNext()) {
					Object[] result = (Object[]) i.next();
					Integer productOrder = (Integer) result[0];
					Integer relatedProductID = (Integer) result[1];
					q2.setInteger("productOrder", productOrder.intValue());
					q2.setInteger("relatedProductID", relatedProductID
							.intValue());
					q2.executeUpdate();
				}
			}
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
		return;
	}

	public Collection loadProductsForCategories(Collection categoryCodes,
			int firstProductID, int lastProductID) throws Exception {
		if (log.isInfoEnabled())
			log.info("Loading products from collection of category codes.");

		Collection items = null;
		Session session = getDaoFactory().startSession();
		try {
			Query q = getNamedQuery(session,
					"product.selectAllFromCategoryCodes");
			q.setParameterList("codes", categoryCodes, StringType.INSTANCE);
			q.setInteger("firstProductID", firstProductID);
			q.setInteger("lastProductID", lastProductID);
			items = new LinkedHashSet(q.list());
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
		return items;
	}

	public Collection loadAllProductsInRange(int firstID, int lastID)
			throws Exception {
		if (log.isInfoEnabled())
			log.info("Loading products within an id range: " + firstID + "-"
					+ lastID);
		Collection items = null;
		Session session = getDaoFactory().startSession();
		try {
			Query q = getNamedQuery(session, "product.selectAllInRange");
			q.setInteger("firstID", firstID);
			q.setInteger("lastID", lastID);
			items = new LinkedHashSet(q.list());
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
		return items;
	}

	public int loadMaxProductSettingID() throws Exception {
		Session session = getDaoFactory().startSession();
		try {
			Query q = getNamedQuery(session, "productSetting.maxProductSettingID");
			Integer maxNum = (Integer) q.uniqueResult();
			if (maxNum == null) {
				return 0;
			}
			return maxNum.intValue();
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
	}

	public Collection loadAllProductSettingsInRange(int firstID, int lastID)
			throws Exception {
		if (log.isInfoEnabled())
			log.info("Loading product settings within an id range: " + firstID + "-"
					+ lastID);
		Collection items = null;
		Session session = getDaoFactory().startSession();
		try {
			Query q = getNamedQuery(session, "productSetting.selectAllInRange");
			q.setInteger("firstID", firstID);
			q.setInteger("lastID", lastID);
			items = new LinkedHashSet(q.list());
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
		return items;
	}

	public int removeCategoryAssociations(String productCode) throws Exception {
		Session session = getDaoFactory().startSession();
		try {
			Query q = getNamedQuery(session, "product.removeCategoryAssocations");
			q.setString("productCode", productCode);
			int result = q.executeUpdate();
			getDaoFactory().commitTransaction();
			return result;
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: ", he);
			throw new DataAccessException(he);
		}
	}

	public int addCategoryAssociation(String productCode, String categoryCode) throws Exception {
		Session session = getDaoFactory().startSession();
		try {
			Query q = getNamedQuery(session, "product.addCategoryAssocation");
			q.setString("productCode", productCode);
			q.setString("categoryCode", categoryCode);
			int result = q.executeUpdate();
			getDaoFactory().commitTransaction();
			return result;
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: ", he);
			throw new DataAccessException(he);
		}
	}

}
